/**
 * @file e32frodo.cpp Frodo Commodore 64 emulator for EPOC32
 *
 * Ported from Win32 to Epoc32 by
 * Alfred E. Heggestad & Jal Panvel
 * Oct 1999 - Jan 2000
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License version 2 as
 *    published by the Free Software Foundation.
 */


#define OFFSET_X 0

#include "sysdeps.h"
#include "e32frodo.h"
#include <e32frodo.rsg>

#include "Prefs.h"
#include "Display.h"
#include "eikinfo.h"
#include <eikon.rsg>
#include <coeutils.h>
#include <apgtask.h>
#include <apgwgnam.h>
#include <eikdll.h>
#include <eikmenup.h>
#include <eikfnlab.h>
#include <gdi.h>

#ifndef __SERIES60__
#include <eikcfdlg.h>
#endif

#if defined(__ER6__)
#include <eikspane.h>
#else
#include <eikmenu.hrh>
#endif // __ER6__

#include "prefsdialog.h"
#include "version"

/// UID of app
#ifdef _UNICODE
const TUid KE32FrodoUid = { 0x10005f47 };
#else
const TUid KE32FrodoUid = { 0x10005f46 };
#endif

/// default snapshot file name
_LIT(KDefaultSnapshot, "C:\\frodo.snap");


//
// EXPORTed functions
//

EXPORT_C CApaApplication* NewApplication()
	{
	ELOG1(_L8("#\n"));
	ELOG2(_L8("# %s\n"), VERSION);
	ELOG1(_L8("#\n"));
	ELOG2(_L8("# please go to %s for updates and more info.\n"), HOMEPAGE);
	ELOG1(_L8("# your bugreports and patches are welcome.\n"));
	ELOG2(_L8("# compiled on %s.\n"), __DATE__);
	ELOG1(_L8("#\n"));
	return new CE32FrodoApplication;
	}

GLDEF_C TInt E32Dll(TDllReason)
	{
	return KErrNone;
	}

////////////////////////////////////////////////////////////////
//
// Application class, CE32FrodoApplication
//
////////////////////////////////////////////////////////////////

TUid CE32FrodoApplication::AppDllUid() const
	{
	return KE32FrodoUid;
	}


CApaDocument* CE32FrodoApplication::CreateDocumentL()
/**
 * Construct the document using its NewL() function, rather 
 * than using new(ELeave), because it requires two-phase
 * construction.
 */
	{
	return CE32FrodoDocument::NewL(*this);
	}


TFileName CE32FrodoApplication::GetDriveAndPath()
/**
 * Return the drive and path to E32FRODO.APP
 */
	{
	TParsePtrC parse(DllName());
	return parse.DriveAndPath();
	}


////////////////////////////////////////////////////////////////
//
// Document class, CE32FrodoDocument
//
////////////////////////////////////////////////////////////////


CE32FrodoDocument::CE32FrodoDocument(CEikApplication& aApp)
/**
 * C'tor
 */
	:CEikDocument(aApp)
	{
	}


CE32FrodoDocument* CE32FrodoDocument::NewL(CEikApplication& aApp)
	{
	CE32FrodoDocument* self = new (ELeave) CE32FrodoDocument(aApp);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}


void CE32FrodoDocument::ConstructL()
	{
	CTOR(CE32FrodoDocument);
	new C64Starter();
	}


CE32FrodoDocument::~CE32FrodoDocument()
/**
 * All resources allocated in ConstructL() must be released in 
 * the destructor.
 */
	{
	DTOR(CE32FrodoDocument);
	CloseSTDLIB();
	}


CEikAppUi* CE32FrodoDocument::CreateAppUiL()
	{
	CEikAppUi* tmp = new (ELeave) CE32FrodoAppUi; 
	iAppUiPtr = (CE32FrodoAppUi*)tmp;
    return tmp;
	}

#if 0
void CE32FrodoDocument::ShowError(char* str1, char* str2)
/**
 * an error has occured. Display it on the screen and the
 * debug port.
 *
 * @param str1 Error string 1
 * @param str2 Error string 2
 */
	{
    TBuf<32> str1d;
    str1d.Copy(TPtrC8((TText8*)str1));
    TBuf<32> str2d;
    str2d.Copy(TPtrC8((TText8*)str2));

	RDebug::Print(_L("%S %S"), &str1d, &str2d );
	CEikInfoDialog* info = new CEikInfoDialog(str1d, str2d);
	if(info)
		{
		TRAPD(ret, info->ExecuteLD(R_EIK_DIALOG_INFO));
		if(ret != KErrNone)
			RDebug::Print(_L("Hmm.") );
		}
	else
		RDebug::Print(_L("Ugg.") );
	}

#endif



////////////////////////////////////////////////////////////////
//
// App UI class, CE32FrodoAppUi
//
////////////////////////////////////////////////////////////////

CE32FrodoAppUi::CE32FrodoAppUi()
/**
 * Constructor
 */
	:iMoveRate(1)
	{
	}


void CE32FrodoAppUi::ConstructL()
/**
 * safe constructor
 */
	{
	CTOR(CE32FrodoAppUi);

/// @todo check why we do this
#if defined(__ER6__)
    //!! This is to disable heap checking fails in debug builds. 
    //!! Heap usage should be investigated and make sure that Frodo do not eat memory when running
    iCoeEnv->DisableExitChecks(ETrue); 
#endif

    BaseConstructL();
    iAppView = new (ELeave) CE32FrodoAppView();

    iAppView->ConstructL(ClientRect());

/// @todo check why we do this
#if defined(__ER6__)
    AddToStackL(iAppView);
#else
	CEikFileNameLabel* filenameLabel=STATIC_CAST(CEikFileNameLabel*, iToolBar->ControlById(ECmdFileName));
	filenameLabel->UpdateL();
#endif

	iLastC64KeyBufIx = 0; /// @todo move to ctor init list
	iJoyKey = 0xff;

	// panics because iFrodo == NULL
#if 0
	Prefs* prefs = E32FrodoDocPtr()->FrodoPtr()->TheC64->ThePrefs;
	__ASSERT_ALWAYS(prefs != NULL, User::Panic(_L("NULL-ptr"), KErrBadHandle) );
	iAppView->iFullScreenMode = !prefs->iShowToolbar;
	ViewToolbar(prefs->iShowToolbar);
#endif
    }


CE32FrodoAppUi::~CE32FrodoAppUi()
/**
 * Dtor
 */
	{
	DTOR(CE32FrodoAppUi);
#ifndef __ER6__
	TRAPD(error,StopHelpL());
#endif
    delete iAppView;
	}


void CE32FrodoAppUi::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane )
/**
 * Dynamically initialises a menu pane. The Uikon framework calls this function,
 * if it is implemented in a menus observer, immediately before the menu pane is
 * activated. Typically this function should inquire the value of application data,
 * and initialise menu items accordingly. This includes dimming menu items, changing
 * their text, setting the state of checked items and radio buttons and dynamically
 * adding items to a menu. 
 *
 * @param aResourceId Resource ID identifying the main menu pane to initialise.
 * @param aMenuPane   In/out. Pointer to the menu pane.
 */
	{
	iAppView->iPauseFromUi = ETrue;
	SetPauseState();
	iAppView->DrawNow();

	switch (aResourceId)
		{
		case R_E32FRODO_TOOLS_MENU:
			break;

		case R_E32FRODO_VIEW_MENU:
			if (iAppView->iFullScreenMode)
				{
				aMenuPane->SetItemButtonState(ECmdFullScreen, EEikMenuItemSymbolOn);
				}
			break;
		}
	}


void CE32FrodoAppUi::SetEmphasis(CCoeControl* /*aMenuControl*/,TBool aEmphasis)
/**
 * called by Eikon when exiting menu (?)
 */
	{
	iAppView->iPauseFromUi = aEmphasis;
	SetPauseState();
	}


void CE32FrodoAppUi::HandleForegroundEventL(TBool aForeground)
/**
 * Handle a foreground event
 */
	{
	iAppView->iPauseFromUi = !aForeground;
	SetPauseState();
#if defined(__ER6__)
	CEikAppUi::HandleForegroundEventL(aForeground);
#endif
	}


static const char* const KCmdLiteralTable[] =
	{
	"ECmdFileName=100",
	"ECmdPrefs=101",
	"ECmdPause=102",
	"ECmdReset=103",
	"ECmdLimit=104",
	"ECmdFullScreen=105",
	"ECmdScreenMoveUp=106",
	"ECmdScreenMoveDown=107",
	"ECmdScreenMoveLeft=108",
	"ECmdScreenMoveRight=109",
	"ECmdScreenMoveCenter=110",
	"ECmdSnapShotSave=111",
	"ECmdSnapShotLoad=112"
	};




//#if ((sizeof(KCmdLiteralTable)/sizeof(KCmdLiteralTable[0])) != (ECmdMax - ECmdMin + 1))
//#error please update KCmdLiteralTable when adding commands in e32frodo.hrh
//#endif


void CE32FrodoAppUi::HandleCommandL(TInt aCommand)
/**
 * Handle commands from the Toolbar
 * (prefs, pause, reset, limit, ...)
 */
	{
	ELOG(if(ECmdMin <= aCommand && aCommand < ECmdMax))
		ELOG3(_L8("handle command '%s' (%d)\n"), KCmdLiteralTable[aCommand-ECmdMin], aCommand);
	ELOG(else)
		ELOG2(_L8("handle command %d\n"), aCommand);

	/**
	 * pre
	 */
	if(ECmdScreenMoveUp <= aCommand && aCommand <= ECmdScreenMoveRight)
		{
		if(aCommand == iLastCommand)
			iMoveRate++;
		else
			iMoveRate = 1;
		}

	/**
	 * handle the command now
	 */
	switch (aCommand)
		{
	case EEikCmdHelpContents:	// Help contents
		{
#ifdef __ER6__
		/// @todo start the web browser here with our html page
		iEikonEnv->InfoMsg(_L("TODO: start browser with \\system\\apps\\e32frodo\\e32frodohtlp.htm") );
#else
		StartHelpL();
#endif
		}
		break;

	case EEikCmdHelpAbout:	// About
		{
		iAppView->iPauseFromUi = ETrue;
		SetPauseState();
		iAppView->DrawNow();

		CEikDialog* aboutDial = new (ELeave) CEikDialog();
		aboutDial->ExecuteLD(R_E32FRODO_ABOUT_DIALOG);

		iAppView->iPauseFromUi = EFalse;
		SetPauseState();
		iAppView->DrawNow();
		}
		break;

    case ECmdFullScreen:
		{
		__ASSERT_DEBUG(E32FrodoDocPtr()->FrodoPtr()->TheC64, User::Panic(_L("NULL-ptr"), KErrBadHandle) );
		Prefs& prefs = E32FrodoDocPtr()->FrodoPtr()->TheC64->ThePrefs;
		prefs.iShowToolbar = !prefs.iShowToolbar;
		iAppView->iFullScreenMode = !prefs.iShowToolbar;

#if defined(__ER6__)
        // View the image on the full screen view or on the normal view
		iEikonEnv->AppUiFactory()->ToolBar()->MakeVisible(!iAppView->iFullScreenMode);
		CEikStatusPane* statusp = iEikonEnv->AppUiFactory()->StatusPane();
		if (statusp)
			statusp->MakeVisible(!iAppView->iFullScreenMode);
        iAppView->SetRect(iEikonEnv->EikAppUi()->ClientRect());

#endif
		ViewToolbar(prefs.iShowToolbar);
        iAppView->UpdateDrawPositions();
        iAppView->DrawNow();
		}
		break;

	case ECmdScreenMoveUp:
		iAppView->iOffset.iY -= iMoveRate;
		break;

	case ECmdScreenMoveDown:
		iAppView->iOffset.iY += iMoveRate;
		break;

	case ECmdScreenMoveLeft:
		iAppView->iOffset.iX -= iMoveRate;
		break;

	case ECmdScreenMoveRight:
		iAppView->iOffset.iX += iMoveRate;
		break;

	case ECmdScreenMoveCenter:
		iAppView->iOffset.SetXY(OFFSET_X, ((iAppView->iScreenSize.iHeight-DISPLAY_Y) / 2) + 1 );
		break;

	case ECmdPrefs:	// Prefs
		{
		iAppView->iPauseFromUi = ETrue;
		SetPauseState();
		iAppView->DrawNow();

		// Open Prefs dialog box
		C64* c64 = E32FrodoDocPtr()->FrodoPtr()->TheC64;

		CPrefsDialog* prefsDial = new (ELeave) CPrefsDialog(c64);
		prefsDial->ExecuteLD(R_TDLG_DIALOG_FOCUS);

		iAppView->iPauseFromUi = EFalse;
		SetPauseState();
		iAppView->DrawNow();
		}
		break;

	case ECmdPause:	// Pause
		{
        iAppView->iPause = !iAppView->iPause;
        SetPauseState();
		if(iAppView->iPause)
			iEikonEnv->InfoMsg(_L("pause on") );
		else
			iEikonEnv->InfoMsg(_L("pause off") );
		}
		break;

	case ECmdReset:	// Reset
		{
		if(E32FrodoDocPtr()->FrodoPtr() == NULL)
			break;

		// Reset the C64
		E32FrodoDocPtr()->FrodoPtr()->TheC64->Reset();
		_LIT(KResetMessage, "reset");
		iEikonEnv->InfoMsg(KResetMessage);
        iAppView->UpdateDrawPositions();
		}
		break;

	case ECmdLimit:	// Toggle Limit
		{
		bool& limit = E32FrodoDocPtr()->FrodoPtr()->TheC64->ThePrefs.LimitSpeed;
		limit = !limit;

		TBuf<32> txt;
		txt.Format(_L("limit %s"), limit ? "on" : "off");
		iEikonEnv->InfoMsg(txt);
		}
		break;

	case EEikCmdExit:
		// stop the emulation and the destroy all the objects. Goodbye
		if(E32FrodoDocPtr()->FrodoPtr())
			{
			E32FrodoDocPtr()->FrodoPtr()->TheC64->Quit();
			delete E32FrodoDocPtr()->FrodoPtr();
			}
		Exit();
		break;

	case ECmdSnapShotSave:
		{
#ifdef __SERIES60__
		_LIT(KNotYet, "Not yet...");
		iEikonEnv->InfoMsg(KNotYet);
#else
		iAppView->iPauseFromUi = ETrue;
		SetPauseState();

		TFileName file;
		file.Copy(KDefaultSnapshot);
		CEikFileSaveAsDialog* save = new CEikFileSaveAsDialog(&file);
		if (save->ExecuteLD(R_EIK_DIALOG_FILE_SAVEAS))
			{
			iCurrentFile.Copy(file);
			iSnapState = ESnapShotSave;
			ELOG2(_L8("snapshot saved as '%S'\n"), &iCurrentFile);
			}

		iAppView->iPauseFromUi = EFalse;
		SetPauseState();
#endif // __SERIES60__
		}
		break;
		  
	case ECmdSnapShotLoad:
		{
#ifdef  __SERIES60__
		_LIT(KNotYet, "Not yet...");
		iEikonEnv->InfoMsg(KNotYet);
#else
		iAppView->iPauseFromUi = ETrue;
		SetPauseState();

		TFileName file;
		file.Copy(KDefaultSnapshot);
		CEikFileOpenDialog* load = new CEikFileOpenDialog(&file);
		if (load->ExecuteLD(R_EIK_DIALOG_FILE_OPEN))
			{
			iCurrentFile.Copy(file);
			iSnapState = ESnapShotLoad;
			ELOG2(_L8("snapshot loaded '%S'\n"), &iCurrentFile);
			}

		iAppView->iPauseFromUi = EFalse;
		SetPauseState();
#endif // __SERIES60__
		}
		break;
		
	default:
		break;

		} // switch(aCommand)


	/**
	 * post
	 */
	if(ECmdScreenMoveUp <= aCommand && aCommand <= ECmdScreenMoveCenter)
		{
		iAppView->UpdateDrawPositions();
		TBuf<64> buf;
		buf.AppendFormat(_L("offset (%d,%d)"), iAppView->iOffset.iX, iAppView->iOffset.iY);
		iEikonEnv->InfoMsg(buf);
		}

	iLastCommand = aCommand;
	}


void CE32FrodoAppUi::OpenFileL(const TDesC& aFileName)
/**
 * called by EIKON framework when user taps on e.g. Frodo snapshot files
 * we could handle more types here if we want, for example prefs files,
 * .d64 .d64.gz .t64 .t64.gz etc...
 */
	{
	TBuf8<256> buf8;
	buf8.Copy(aFileName);
	ELOG2(_L8("OpenFileL [doc=%S]\n"), &buf8);

	if (ConeUtils::FileExists(aFileName))
		{
		iCurrentFile.Copy(buf8);
		iSnapState = ESnapShotLoad;
		}
	
	}


 

TBool CE32FrodoAppUi::ProcessCommandParametersL(TApaCommand aCommand, TFileName& aDocumentName,const TDesC8& /*aTail*/)
/**
 * called by EIKON framework during application startup. The variable aDocumentName contains the name of the file
 * that the user tapped on in shell.
 *
 * Some helpful info here:
 *
 * http://www2.epocworld.com/kbcppf.nsf/ce9a3a52e9d969788025670e00429dbd/df4f1b9d4146e6d3802569f3005eabf4?OpenDocument
 *
 * Headline:     Why does CMyAppUi::ProcessCommandParametersL() not seem to get called on V6 of the Symbian Platform?
 * Category:     Application architecture
 * EPOC Release: ER6
 * KeyWords:     Application architecture, Porting, ProcessCommandParametersL, Document, File names
 * Report:       Why does CMyAppUi::ProcessCommandParametersL() not seem to get called on V6 of the Symbian Platform?
 * Answer:  
 *
 * This can often catch you out when porting code from ER5 to V6. The function: 
 *
 * TBool CMyAppUi::ProcessCommandParametersL(TApaCommand aCommand,TFileName& aDocumentName,const TDesC&)
 * 
 * on ER5 must be coded as:
 *
 * TBool CMyAppUi::ProcessCommandParametersL(TApaCommand aCommand,TFileName& aDocumentName,const TDesC8& )
 * 
 * on V6 otherwise it will not get called.
 *
 */
	{
	TBuf8<256> buf8;
	buf8.Copy(aDocumentName);
	ELOG3(_L8("ProcessCommandParametersL [cmd=%d, doc=%S]\n"), aCommand, &buf8);

	if(aCommand==EApaCommandOpen && aDocumentName.Length())
		{
		iCurrentFile.Copy(buf8);
		iSnapState = ESnapShotLoad;
		}

    // This prevents unnecessary zero length document files from being created
    aDocumentName.Zero();
/*
0	EApaCommandOpen,
1	EApaCommandCreate,
2	EApaCommandRun,
3	EApaCommandBackground,
*/
	return CEikAppUi::ProcessCommandParametersL(aCommand, aDocumentName);
	}


#if defined(__ER6__)
TKeyResponse
#else
void
#endif
CE32FrodoAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
/**
 * Handle a key event
 *
 * @todo simplify this function a bit
 */
	{
	iAppView->iBgUpdate = ETrue;

    if (aKeyEvent.iModifiers&EModifierCapsLock)
        {
        TInt joyDelta=0;
        switch(aKeyEvent.iScanCode)
            {
            case EStdKeyLeftCtrl:
                joyDelta=0x10;
                break;

            case EStdKeyUpArrow:
                joyDelta=0x1;
                break;

            case EStdKeyDownArrow:
                joyDelta=0x2;
                break;

            case EStdKeyLeftArrow:
                joyDelta=0x4;
                break;

            case EStdKeyRightArrow:
                joyDelta=0x8;
                break;
            }

        switch( aType )
            {
            case EEventKeyDown:
                iJoyKey&=0xff^joyDelta;
                break;
            case EEventKeyUp:
                iJoyKey|=joyDelta;
                break;
            default:
                break;

            }

        if (joyDelta!=0)
            {
#if defined (__ER6__)
			return EKeyWasConsumed;
#else
			return;
#endif
            }
        }

	TInt c64Key;

	// if c64key == -1 then skip
	if (aType == EEventKey || aType == EEventKeyDown)
		{
		c64Key = C64Display::VirtKey2C64(aKeyEvent, iJoyKey);

		if(iJoyKey != 0xff)

		if (c64Key < 0 || c64Key >= 256)	// joykey
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif

		if (keystate[c64Key])
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif

		// store key in buf
		if(iLastC64KeyBufIx >= KKeyBufSize)
			{
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif
			}
		else iLastC64KeyBuf[iLastC64KeyBufIx++] = c64Key;
		
		keystate[c64Key] = 1;
		int c64_byte = c64Key >> 3;
		int c64_bit = c64Key & 7;
		int shifted = c64Key & 128;
		c64_byte &= 7;
		if (shifted) {
			key_matrix[6] &= 0xef;
			rev_matrix[4] &= 0xbf;
			}
		key_matrix[c64_byte] &= ~(1 << c64_bit);
		rev_matrix[c64_bit] &= ~(1 << c64_byte);


		}
	else if (aType == EEventKeyUp)
		{

		iJoyKey = 0xff;

		if(iLastC64KeyBufIx <= 0)
			{
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif
			}
		else c64Key = iLastC64KeyBuf[--iLastC64KeyBufIx];

		if (c64Key < 0 || c64Key >= 256) // joykey
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif
		if (!keystate[c64Key])
#if defined (__ER6__)
			return EKeyWasNotConsumed;
#else
			return;
#endif

		keystate[c64Key] = 0;
		int c64_byte = c64Key >> 3;
		int c64_bit = c64Key & 7;
		int shifted = c64Key & 128;
		c64_byte &= 7;
		if (shifted) {
			key_matrix[6] |= 0x10;
			rev_matrix[4] |= 0x40;
			}
		key_matrix[c64_byte] |= (1 << c64_bit);
		rev_matrix[c64_bit] |= (1 << c64_byte);
		}

#if defined (__ER6__)
	return EKeyWasNotConsumed;
#endif
	} // end, CE32FrodoAppUi::HandleKeyEventL


void CE32FrodoAppUi::SetPauseState()
/**
 * set the pause state of the emulation
 */
	{
	if (E32FrodoDocPtr() && E32FrodoDocPtr()->FrodoPtr() && E32FrodoDocPtr()->FrodoPtr()->TheC64)
		{
		TBool paused = E32FrodoDocPtr()->FrodoPtr()->TheC64->Paused();
		TBool mustPause = iAppView->iPause || iAppView->iPauseFromUi;
		if ((mustPause && !paused) || (!mustPause && paused))
			E32FrodoDocPtr()->FrodoPtr()->TheC64->TogglePause();
		}
	}


//
// copied from helloeik.cpp
//

#ifndef __ER6__

_LIT(KHelp, ".hlp");
_LIT(KMatch, "- %S");

void CE32FrodoAppUi::StartHelpL()
	{
	
	// get the help filename
	TFileName appFileName = Application()->AppFullName();
	TParse fileNameParser;
	fileNameParser.SetNoWild(KHelp, &appFileName, NULL);
	TPtrC helpFileFullName = fileNameParser.FullName();
	if (!ConeUtils::FileExists(helpFileFullName))
		User::Leave(KErrNotFound);
	// scan through open programs
	TPtrC helpFileName=fileNameParser.Name();
	HBufC* matchString=HBufC::NewLC(2 + helpFileName.Length());
	matchString->Des().Format(KMatch , &helpFileName);
	TApaTaskList taskList (iEikonEnv->WsSession());
	TInt numGroups = iEikonEnv->WsSession().NumWindowGroups(0);
	for (TInt index=0; index < numGroups; index++)
		{
		// get task information
		TApaTask* task= new (ELeave) TApaTask(taskList.FindByPos(index));
		CleanupStack::PushL(task);
		CApaWindowGroupName* wgName=
				CApaWindowGroupName::NewLC(iEikonEnv->WsSession(),task->WgId());
		TUid uid = wgName->AppUid();
		TPtrC taskCaption = wgName->Caption();
		// check help app, and with right caption
		TBool found =
				uid.iUid == 0x10000171 && // help app
				taskCaption.FindF(*matchString)>=0; // matching caption
		// bring to foreground if we found it
		if (found) 
			{
			task->BringToForeground();
			CleanupStack::PopAndDestroy(3); // wgName, task, matchString
			return; // exit procedure
			}
		CleanupStack::PopAndDestroy(2); // wgName, task
		}
	CleanupStack::PopAndDestroy(); // matchString
	// if not found, then open the help database
	EikDll::StartDocL(helpFileFullName);
	User::After(1000000); // pause 1 second to give a decent time to start
	}


void CE32FrodoAppUi::StopHelpL()
	{
	// get the help filename
	TFileName appFileName = Application()->AppFullName();
	TParse fileNameParser;
	fileNameParser.SetNoWild(KHelp, &appFileName, NULL);
	TPtrC helpFileFullName = fileNameParser.FullName();
	if (!ConeUtils::FileExists(helpFileFullName))
		return;
	// scan through open programs
	TPtrC helpFileName=fileNameParser.Name();
	HBufC* matchString=HBufC::NewLC(2 + helpFileName.Length());
	matchString->Des().Format(KMatch, &helpFileName);
	TApaTaskList taskList (iEikonEnv->WsSession());
	TInt numGroups = iEikonEnv->WsSession().NumWindowGroups(0);
	for (TInt index=0; index < numGroups; index++)
		{
		// get task information
		TApaTask* task= new (ELeave) TApaTask(taskList.FindByPos(index));
		CleanupStack::PushL(task);
		CApaWindowGroupName* wgName=
				CApaWindowGroupName::NewLC(iEikonEnv->WsSession(),task->WgId());
		TUid uid = wgName->AppUid();
		TPtrC taskCaption = wgName->Caption();
		// check help app, and with right caption
		TBool found =
				uid.iUid == 0x10000171 && // help app
				taskCaption.FindF(*matchString)>=0; // matching caption
		// bring to foreground if we found it
		if (found) 
			{
			task->EndTask();
			CleanupStack::PopAndDestroy(3); // wgName, task, matchString
			return; // exit procedure
			}
		CleanupStack::PopAndDestroy(2); // wgName, task
		}
	CleanupStack::PopAndDestroy(); // matchString
	}

#endif // !__ER6__


void CE32FrodoAppUi::ViewToolbar(TBool aView)
/**
 * set toolbar visible or not
 */
	{
#if defined(__ER6__)
	iAppView->SetRect(ClientRect());
	(void)aView;
#else
    iToolBar->MakeVisible(aView);
	TRAPD(ret, iAppView->SetRectL(ClientRect()));
	(void)ret;
#endif // __ER6__
	}


//
// Application view class, CE32FrodoAppView
//


void CE32FrodoAppView::ConstructL(const TRect& aRect)
/**
 * safe constructor
 */
	{
	CTOR(CE32FrodoAppView);

	CreateWindowL();
#if defined(__ER6__)
	SetRect(aRect);
#else
	SetRectL(aRect);
#endif
	ActivateL();
	iBgUpdate = ETrue;

#if _DEBUG	
	static const char* const KDispModeTable[] =
		{
		"None",
		"Gray2",
		"Gray4",
		"Gray16",
		"Gray256",
		"Color16", 
		"Color256",
		"Color64K",
		"Color16M",
		"Rgb",
		"Color4K"
		};
#endif // _DEBUG

	//
	// check graphic capabilities (colors/grey)
	//

	iScreenSize = iCoeEnv->ScreenDevice()->SizeInPixels();
	ELOG3(_L8("Screen size is %d x %d\n"), iScreenSize.iWidth, iScreenSize.iHeight);

	iDispMode = iCoeEnv->ScreenDevice()->DisplayMode();
	ELOG3(_L8("Display mode is '%s' (%d)\n"), (size_t)iDispMode > sizeof(KDispModeTable)/sizeof(KDispModeTable[0]) 
		? "out of bounds"
		: KDispModeTable[iDispMode], iDispMode );
	
	iIsColorMachine = (iDispMode >= EColor16);
	ELOG2(_L8("Color machine: %s\n"), iIsColorMachine ? "Yes" : "No" );
	ELOG2(_L8("COLOR_DEPTH macro is %d\n"), COLOR_DEPTH);

	// Select the right display mode
#if COLOR_DEPTH == 4
	iDispMode = iIsColorMachine ? EColor16 : EGray16;
#elif COLOR_DEPTH == 8
	iDispMode = iIsColorMachine ? EColor256 : EGray256;
#elif COLOR_DEPTH == 12
	Not supported yet!
#elif COLOR_DEPTH == 24
	Not supported yet!
#endif

	TInt ret = KErrNone;

#if defined(__ER5__)
	// set the new display type to 16 colors/grey
	RDrawableWindow* win = DrawableWindow();
	ret = win->SetRequiredDisplayMode(iDispMode);
	if(ret != KErrNone)
		{
		ELOG2(_L8("could not set disp mode [ret=%d]"), ret);
		User::Leave(ret);
		}
#endif // __ER5__
	
	// Create the BitMap

#if COLOR_DEPTH == 4
	ret = iTheBitMap.Create(TSize(DISPLAY_X*2, DISPLAY_Y), iDispMode);	//AEH 991203
#elif COLOR_DEPTH == 8

#if defined(__ER6__) /// @todo really should be Crystal 6.0

    iMyBufferSize = TSize(DISPLAY_X, DISPLAY_Y);
    iMyScreenBuffer = new (ELeave) TUint8[iMyBufferSize.iWidth * iMyBufferSize.iHeight];
    
    iMyColorTable =  new (ELeave) TUint16[256];
    for (TInt i=0; i<256; i++)
		iMyColorTable[i] = 0;

    iMyColorTable[0x00] = 0x000; // black
    iMyColorTable[0x0f] = 0xfff; // white
    iMyColorTable[0x02] = 0xa00; // dark red
    iMyColorTable[0x0a] = 0x0fd; // cyan
    iMyColorTable[0x08] = 0xd0d; // purple
    iMyColorTable[0x04] = 0x4d4; // dark green
    iMyColorTable[0x0c] = 0x10a; // dark blue
    iMyColorTable[0x06] = 0xff0; // yellow
    iMyColorTable[0x03] = 0xb50; // brown
    iMyColorTable[0x0b] = 0x630; // dark brown
    iMyColorTable[0x05] = 0xf66; // light red
    iMyColorTable[0x01] = 0x444; // dark grey
    iMyColorTable[0x0d] = 0x888; // med grey
    iMyColorTable[0x07] = 0x6f6; // light green
    iMyColorTable[0x09] = 0x77f; // light blue
    iMyColorTable[0x0e] = 0xccc; // light grey

#else // __ER6__

	// for ER5 - NetBook/Series7
	ret = iTheBitMap.Create(TSize(DISPLAY_X, DISPLAY_Y), iDispMode);	//AEH 991203

#endif // __ER6__

#elif // COLOR_DEPTH
error
#endif // COLOR_DEPTH

	// did it work?
	if(ret != KErrNone)
		{
		ELOG2(_L8("could not create BitMap [ret=%d]"), ret);
		User::Leave(ret);
		}

    _LIT(KFrodoFont, "Swiss");
    TFontSpec spec(KFrodoFont, 213);
    iSpeedFont = iCoeEnv->CreateScreenFontL(spec);

	// calculate screensize and offsets
	iOffset.SetXY(OFFSET_X, ((iScreenSize.iHeight-DISPLAY_Y) / 2) +1) ;

	// rectangle for inner C64 screen
	iC64screen.SetRect(TPoint(0x20, 0x23), TPoint(0x160, 0xeb));

#if defined(__ER6__) /// @todo should be Crystal
#ifdef __WINS__

    iWinsScreenBitmap = new (ELeave) CFbsBitmap;
    ret = iWinsScreenBitmap->Create(TSize(DISPLAY_X, 200/*!!DISPLAY_X, DISPLAY_Y*/), EColor4K);
	if(ret != KErrNone)
		{
		ELOG2(_L8("could not create BitMap [ret=%d]"), ret);
		User::Leave(ret);
		}

#endif // __WINS__
#endif // __ER6__

	/// @todo move to ctor init list
	iFullScreenMode = EFalse;
	iPauseFromUi = EFalse;
	iPause = EFalse;

	UpdateDrawPositions();

	} // end, CE32FrodoAppView::ConstructL



CE32FrodoAppView::~CE32FrodoAppView()
/**
 * Destructor
 */
	{
	DTOR(CE32FrodoAppView);

	// release fonts
	iCoeEnv->ReleaseScreenFont(iSpeedFont);

#if defined(__ER6__)
    //!! Delete cfbsbitmap
#ifdef __WINS__
    delete iWinsScreenBitmap;
#endif
    delete[] iMyColorTable;
    delete[] iMyScreenBuffer;
#endif // __ER6__

	}


void CE32FrodoAppView::FocusChanged(TDrawNow aDrawNow)
/**
 * focus was changed - update pause state 
 */
	{
    iPauseFromUi = !IsFocused();
    CCoeControl::FocusChanged(aDrawNow);
    }


 
//
// The following patch from mooz@welho.com for Psion Revo
//

void CE32FrodoAppView::HandlePointerEventL(const TPointerEvent& aPointerEvent)
/**
 * This function is called when the user taps the screen in the application view.
 */
	{
	ELOG3(_L8("GUI: pointer event [x=%d, y=%d]\n"),
		aPointerEvent.iPosition.iX,
		aPointerEvent.iPosition.iY);

/* Added for revo by mooz */

	// change offset
	iOffset.iX = aPointerEvent.iPosition.iX - (DISPLAY_X/2);
	iOffset.iY = aPointerEvent.iPosition.iY - (DISPLAY_Y/2);

	iBgUpdate = ETrue;
	UpdateDrawPositions();
	}


void CE32FrodoAppView::DrawBuf(bool bg_update)
/**
 * DrawBuf - using bitmaps
 * Called by VBlank and blits the bitmap to the screen
 *
 * if bg_update is TRUE then the whole bitmap is blitted
 * otherwise, only the inner part is drawn.
 */
	{

	//
	// keep the emulation running but dont update the screen
	//
//	if(iAppView->iPauseFromUi)
//		return;

#if defined(__ER6__)

	DrawDirect(bg_update);

#else // __ER6__

	if(bg_update)
		iBgUpdate = ETrue;

	// Activate Graphics Context and get system
	ActivateGc();
	CWindowGc& gc = SystemGc();

	if(iBgUpdate)	// draw the whole screen
		{
		// Draw the bitmap here
		gc.BitBlt(iOffset, &iTheBitMap);
		iBgUpdate = EFalse;
		}
	else	// only draw inner screen
		{
		gc.BitBlt(iOffset+TPoint(0x20, 0x23), &iTheBitMap, iC64screen );
		}

	// Deactivate Graphics Context
	DeactivateGc();

#endif // __ER6__

	} 


void CE32FrodoAppView::UpdateDrawPositions()
	{

#if defined(__ER6__)

	iMyBufferAddress = iMyScreenBuffer;
	iMyBufferAddress2 = iMyScreenBuffer;

	TSize screenSize;

    TPckgBuf<TScreenInfoV01> info;
	UserSvr::ScreenInfo(info);
	screenSize = info().iScreenSize;

#ifndef __WINS__
	iScreenAddress = (TUint16*)(info().iScreenAddress);
	iScreenAddress[0] = TUint16(2 << 12);
	iScreenAddress += 16;
#else
    iScreenAddress = (TUint16*)iWinsScreenBitmap->DataAddress();
#endif
    iScreenAddress2 = iScreenAddress;  

    // Get the drawable area.
    // Start with the whole screen
    iDrawableArea = TRect(TPoint(0,0), screenSize);
    // Shrink area according to view offset
    iDrawableArea.iTl += iOffset + PositionRelativeToScreen();
    // Limit area to the window area
    iDrawableArea.Intersection(TRect(PositionRelativeToScreen(), Size()));
    // Bitmap area
    TRect bitmapArea(iOffset + PositionRelativeToScreen(), iMyBufferSize);
    TRect bitmapArea2(bitmapArea.iTl + iC64screen.iTl, iC64screen.Size()); //Inner bitmap

    // *** Calc values for outer bitmap drawing

    iDrawableArea.Intersection(bitmapArea);
    iDrawableArea.Intersection(TRect(screenSize));
    TInt screenStartAddress = iDrawableArea.iTl.iY * screenSize.iWidth + iDrawableArea.iTl.iX;
    iScreenAddress += screenStartAddress;

    TPoint bmStartPixel;
    bmStartPixel.iX = Max(0, (iDrawableArea.iTl.iX - bitmapArea.iTl.iX));
    bmStartPixel.iY = Max(0, (iDrawableArea.iTl.iY - bitmapArea.iTl.iY));
    TInt bmStartAddress = bmStartPixel.iY * iMyBufferSize.iWidth + bmStartPixel.iX;
    iMyBufferAddress += bmStartAddress;

    iJumpInPixelsInScreen = screenSize.iWidth - iDrawableArea.Width(); 
    TRect boundingRect = bitmapArea;
    boundingRect.BoundingRect(iDrawableArea);
    iJumpInPixelsInBuffer = boundingRect.Width() - iDrawableArea.Width(); 

    // *** Calc values for inner bitmap drawing

    iDrawableArea2 = iDrawableArea;
    iDrawableArea2.Intersection(bitmapArea2);
    TInt screenStartAddress2 = iDrawableArea2.iTl.iY * screenSize.iWidth + iDrawableArea2.iTl.iX;
    iScreenAddress2 += screenStartAddress2;

    TPoint bmStartPixel2;
    bmStartPixel2.iX = Max(0, (iDrawableArea2.iTl.iX - bitmapArea2.iTl.iX));
    bmStartPixel2.iY = Max(0, (iDrawableArea2.iTl.iY - bitmapArea2.iTl.iY));
    TInt bmStartAddress2 = bmStartPixel2.iY * iMyBufferSize.iWidth + bmStartPixel2.iX;
    iMyBufferAddress2 += bmStartAddress2;

    iJumpInPixelsInScreen2 = screenSize.iWidth - iDrawableArea2.Width(); 
    TRect boundingRect2 = bitmapArea2;
    boundingRect.BoundingRect(iDrawableArea2);
    iJumpInPixelsInBuffer2 = boundingRect2.Width() - iDrawableArea2.Width(); 

#else // __ER6__

/// @todo implement this for ER5

#endif // __ER6__

	// clear the screen
	ActivateGc();
	CWindowGc& gc = SystemGc();
	gc.Clear(TRect(0, 0, iScreenSize.iWidth, iScreenSize.iHeight));
	DeactivateGc();
    }


#if defined (__ER6__)

void CE32FrodoAppView::DrawDirect(bool bg_update)
/**
* Draws directly to the screen
* 
* This could be optimised if Mem::Copy can be used for copying a scanline. This
* is not possible until Frodo writes 4k values to the buffer. Now it writes 
* 256 color values or 16 color values.
*
*/
	{
	TRect drawableArea;
	TUint8* myBufferAddress;
	TUint16* screenAddress;
	TInt jumpInPixelsInScreen;
	TInt jumpInPixelsInBuffer;

	if(bg_update)
		{
		drawableArea = iDrawableArea;
		myBufferAddress = iMyBufferAddress;
		screenAddress = iScreenAddress;
		jumpInPixelsInScreen = iJumpInPixelsInScreen;
		jumpInPixelsInBuffer = iJumpInPixelsInBuffer;
		}
	else
		{
		drawableArea = iDrawableArea2;
		myBufferAddress = iMyBufferAddress2;
		screenAddress = iScreenAddress2;
		jumpInPixelsInScreen = iJumpInPixelsInScreen2;
		jumpInPixelsInBuffer = iJumpInPixelsInBuffer2;
		}

#ifdef __WINS__
	TBitmapUtil bmUtil(iWinsScreenBitmap);
	bmUtil.Begin(TPoint(0,0));
	drawableArea.Move(-PositionRelativeToScreen());
#endif
	
	for(TInt j = drawableArea.iTl.iY; j < drawableArea.iBr.iY; j++)
	    {
		for(TInt i = drawableArea.iTl.iX; i < drawableArea.iBr.iX; i++) 
		    {
			
#ifdef __WINS__
			bmUtil.SetPos(TPoint(i,j));
			bmUtil.SetPixel(iMyColorTable[*myBufferAddress++]);
#else
			*screenAddress++ = iMyColorTable[*myBufferAddress++];
#endif
		    }
		screenAddress += jumpInPixelsInScreen;
		myBufferAddress += jumpInPixelsInBuffer;
	    }
	
#ifdef __WINS__
	
	bmUtil.End();
	
	// Activate Graphics Context and get system
	ActivateGc();
	CWindowGc& gc = SystemGc();
	TRect rect(iWinsScreenBitmap->SizeInPixels());
	//rect.Move(PositionRelativeToScreen());
	gc.BitBlt(TPoint(0,0), iWinsScreenBitmap, rect);
	DeactivateGc();
	
#endif
	}

#endif // (__ER6__)


void CE32FrodoAppView::DrawSpeed(int speed, int max_speed)
	{

#define SPEED_TEXT_POS	TPoint(395, 20)

	// Deactivate Graphics Context and get Windows Context and clear
	ActivateGc();
	CWindowGc& gc = SystemGc();
	gc.Clear(
		TRect
		(
		SPEED_TEXT_POS.iX+30,
		SPEED_TEXT_POS.iY-15,
		SPEED_TEXT_POS.iX+100,
		SPEED_TEXT_POS.iY+30
		) 
		);

	// set color for text
	gc.SetPenColor( TRgb::Color16(0) );

    gc.UseFont(iSpeedFont);

	// format speed info and print it
	TBuf<64> txt;
	txt.Format(_L("spd: %d%c"), speed, '%');
	gc.DrawText(txt, SPEED_TEXT_POS);

	txt.Format(_L("max: %d%c"), max_speed, '%');
	gc.DrawText(txt, SPEED_TEXT_POS + TPoint(0, 20) );

	// Deactivate Graphics Context
	DeactivateGc();

	} // end, CE32FrodoAppView::DrawSpeed


//
// class C64Starter
//
// This class starts the Emulation asynchronously
//
C64Starter::C64Starter()
	:CActive(EPriorityLow)
	{
	CActiveScheduler::Add(this);
	SetActive();
	TRequestStatus* ref = &iStatus;
	User::RequestComplete(ref, KErrNone);
	}

void C64Starter::RunL()
	{
	CE32FrodoDocument* doc = (CE32FrodoDocument*)CEikonEnv::Static()->EikAppUi()->Document();

	if (doc->iFrodo==NULL)
		{
		doc->iFrodo = Frodo::NewL(doc);
		}

	if(doc->FrodoPtr() != NULL)
		{
		doc->FrodoPtr()->StartC64();
		}

	Deque();
	delete this;
	}

void C64Starter::DoCancel()
	{
	}


// EOF - E32FRODO.CPP
