/**
 * @file e32frodo.h Header file for the Eikon Applications part of 
 *                  Frodo C64 emulator for EPOC32
 *
 *  (c) 1999-2001 Alfred E. Heggestad & Jal Panvel
 */

#ifndef __E32FRODO_H
#define __E32FRODO_H

#include <coecntrl.h>
#include <coeccntx.h>
#include <coemain.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <eikenv.h>
#include <eiktbar.h>

#if defined(__ER6__)
#include <eikfnlab.h>
#else
#include <eikcmds.hrh>
#endif

#include "e32frodo.hrh"
#include "main.h"

/**
 * defines the states in the (simple) snapshot state machine
 */
enum TSnapShotState
	{
	ESnapShotIdle,
	ESnapShotSave,
	ESnapShotLoad
	};


class CE32FrodoDocument;
class CE32FrodoAppUi;


/**
 * class CE32FrodoAppView - this class handles the output
 */
class CE32FrodoAppView : public CCoeControl, public MCoeControlBrushContext
	{
	friend class CE32FrodoAppUi;

public:
	void ConstructL(const TRect& aRect);
	~CE32FrodoAppView();
	// aeh
    void UpdateDrawPositions();
	void DrawBuf(bool bg_update);
#if defined (__ER6__)
	void DrawDirect(bool bg_update);
#endif // __ER6__
	void DrawSpeed(int speed, int max_speed);

private: // from CCoeControl
    void FocusChanged(TDrawNow aDrawNow);
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
private:
	TDisplayMode iDispMode;
public:
#if defined (__ER6__)

#ifdef __WINS__
    CFbsBitmap* iWinsScreenBitmap;
#endif
	TUint8* iMyScreenBuffer;
    TSize iMyBufferSize;
    TUint16* iMyColorTable;
#else // __ER6__
	CFbsBitmap iTheBitMap;
#endif // __ER6__

private:
	CFbsFont* iSpeedFont;
	TInt iDrawCount;
	TBool iIsColorMachine;	  ///< true if machine supports colors
	TRect iC64screen;
	TPoint iOffset;           ///< X/Y offset from top-left corner
	TBool iBgUpdate;          ///< true if we need to update the background
	TBool iFullScreenMode;    ///< ETrue if using the full screen
	TBool iPauseFromUi;
	TBool iPause;
	TSize iScreenSize;        ///< size of the screen

#if defined(__ER6__)
	TRect iDrawableArea;
	TUint8* iMyBufferAddress;
	TUint16* iScreenAddress;
	TInt iJumpInPixelsInScreen;
	TInt iJumpInPixelsInBuffer;
	TRect iDrawableArea2;
	TUint8* iMyBufferAddress2;
	TUint16* iScreenAddress2;
	TInt iJumpInPixelsInScreen2;
	TInt iJumpInPixelsInBuffer2;
#endif // __ER6__
	};


/**
 * CE32FrodoAppUi - this class handles the input
 */
class CE32FrodoAppUi : public CEikAppUi
	{
public:
	void ConstructL();
	CE32FrodoAppUi();
	~CE32FrodoAppUi();
    void SetPauseState();
private:
	// from MEikMenuObserver
    virtual void DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane);
	virtual void SetEmphasis(CCoeControl* aMenuControl,TBool aEmphasis);
	// from CEikAppUi
    virtual void HandleForegroundEventL(TBool aForeground);
	virtual void HandleCommandL(TInt aCommand);
	virtual void OpenFileL(const TDesC& aFileName);
	virtual TBool ProcessCommandParametersL(TApaCommand aCommand, TFileName& aDocumentName,const TDesC8& /*aTail*/);

#if defined(__ER6__)
	TKeyResponse
#else
void
#endif
	HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
	void StartHelpL();
	void StopHelpL();
public:
	inline CE32FrodoDocument* E32FrodoDocPtr() {return (CE32FrodoDocument*)Document();}

	CE32FrodoAppView* iAppView;
	void ViewToolbar(TBool aView);

public:
	// Keyboard arrays
	int keystate[256];
	TUint8 rev_matrix[8];
	TUint8 key_matrix[8];
	TUint8 iJoyKey;

	TBuf8<256> iCurrentFile;             ///< contains the filename (e.g. snapshot, prefs, d64, etc)
	TSnapShotState iSnapState;           ///< contains the state of snap shotting (idle, load, save)

private:
#define KKeyBufSize	0x10

	TInt iLastC64KeyBuf[KKeyBufSize];
	TInt iLastC64KeyBufIx;
	TInt iLastCommand;                   ///< last command that was handled
	TInt iMoveRate;                      ///< number of pixels to move the screen
	};


/**
 * CE32FrodoDocument - this class handles the content of the App
 */
class CE32FrodoDocument : public CEikDocument
	{
public:
	// construct/destruct
	CE32FrodoDocument(CEikApplication& aApp);
	~CE32FrodoDocument();
	static CE32FrodoDocument* NewL(CEikApplication& aApp);
	void ConstructL();
	inline Frodo* FrodoPtr(){return iFrodo;}
	void ShowError(char *str1, char *str2);

public:
	CE32FrodoAppUi* iAppUiPtr;
private: // from CEikDocument
	CEikAppUi* CreateAppUiL();
public:
	Frodo* iFrodo;
	};

//
// CE32FrodoApplication
//

class CE32FrodoApplication : public CEikApplication
	{
public:
	TFileName GetDriveAndPath();
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};


/**
 * this class starts the Frodo Emulator
 * it is an active object which sets itself
 * active, starts Frodo emulator and then deletes itself & dies.
 */
class C64Starter : public CActive
	{
public:
	C64Starter();
private:
	virtual void DoCancel();
	virtual void RunL();
	};

#endif // __E32FRODO_H
