// PREFSDIALOG.CPP
//
// Copyright PanSoftware Ltd 2000
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//

/**
 * @file prefsdialog.cpp
 *
 * Code for the Preference dialog box
 */

#include <basched.h>
#include <eikenv.h>
#include <eikdialg.h>
#include <eikbtpan.h>
#include <eiklbbut.h>
#include <eikcmbut.h>
#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>

#ifndef __SERIES60__
#include <eikchlst.h> // CEikChoiceList
#endif

#include <eikseced.h>
#include <eikprogi.h>
#include <eikrted.h>
#include <eikgted.h>
#include <eikdlgtb.h>
#include <txtrich.h>
#include <txtglobl.h>
#include <eikedwin.h>
#include <eikon.rsg>
#include <eikclb.h>
#include <eikclbd.h>
#include <badesca.h>
#include <eiktxlbx.h>
#include <eikfctry.h>
#include <w32std.h>
#include <e32keys.h>

#if defined(__CRYSTAL__)
#include <eikbtgpc.h>
#include <ckndgopn.h>
#include <eikmfne.h>
#elif defined(__SERIES60__)
/// @todo placeholder
#elif defined(__QUARTZ__)
#include <eikbtgpc.h>
#include <eikfbrow.h>   // CEikFileBrowserDialog
#else
#include <eikmfne.h>
#include <eikdialg.hrh>
#include <eikcmds.hrh>
#include <eikedwin.hrh>
#include <eiktxtut.h>
#include <eikchkbx.h>   // CEikCheckBox
#include <eikdclbm.h>   // CDirContentsListBoxModel
#include <eikfbrow.h>   // CEikFileBrowserDialog
#endif

#include "e32frodo.hrh"
#include <e32frodo.rsg>
#include "prefsdialog.h"


//
// CPrefsDialog implementation
//


CPrefsDialog::CPrefsDialog(C64* aC64)
/**
 * Constructor
 */
	:iC64(aC64)
	,iOrigPrefs(&(aC64->ThePrefs))
	{
	__CHECK_NULL(aC64);

	iPrefs = new Prefs();
	*iPrefs = *iOrigPrefs;

	CTOR(CPrefsDialog);
	}


CPrefsDialog::~CPrefsDialog()
/**
 * Destructor
 */
	{
	delete iPrefs;
	DTOR(CPrefsDialog);
	}


void CPrefsDialog::PreLayoutDynInitL()
	{

	// 1541
	for(TInt i=0;i<4;i++)
		{
		TText8* buf= (TText8*)iPrefs->DrivePath[i];
		TInt bufLen=TPtrC8(buf).Length();
		TInt maxBufLen = sizeof(iPrefs->DrivePath[i]);
		TPtr8 drivePath(buf,bufLen,maxBufLen);
        TFileName drivePath2;
        drivePath2.Copy(drivePath);
		STATIC_CAST(CEikEdwin*,Control(EDlg1541Drive8+i))->SetTextL(&drivePath2);
		}

#if defined(__CRYSTAL__)

	// 1541
	SetChoiceListCurrentItem(EDlg1541Emul1541Proc, iPrefs->Emul1541Proc);

	// Joystick
	SetChoiceListCurrentItem(EDlgJoystick1On, iPrefs->Joystick1On);
	SetChoiceListCurrentItem(EDlgJoystick2On, iPrefs->Joystick2On);

	// VIC
	SetNumberEditorValue(EDlgVICSkipFrames, iPrefs->SkipFrames);
	SetChoiceListCurrentItem(EDlgVICSpritesOn, iPrefs->SpritesOn);
	SetChoiceListCurrentItem(EDlgVICSpriteColl, iPrefs->SpriteCollisions);
	
	// SID
/**
   @todo enable SID

	STATIC_CAST(CEikChoiceList*,Control(EDlgSIDType))->SetCurrentItem(iPrefs->SIDType);
	SetChoiceListCurrentItem(EDlgSIDFilters, iPrefs->SIDFilters);
	SetNumberEditorValue(EDlgSIDLatencyMin, iPrefs->LatencyMin);
	SetNumberEditorValue(EDlgSIDLatencyMax, iPrefs->LatencyMax);
	SetNumberEditorValue(EDlgSIDLatencyAvg, iPrefs->LatencyAvg);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDVolume))->SetNumber(iPrefs->iVolume);
*/

	// CPU
	SetNumberEditorValue(EDlgCPUNormalCycles, iPrefs->NormalCycles);
	SetNumberEditorValue(EDlgCPUBadLineCycles, iPrefs->BadLineCycles);
	SetNumberEditorValue(EDlgCPUCIACycles, iPrefs->CIACycles);
	SetNumberEditorValue(EDlgCPUFloppyCycles, iPrefs->FloppyCycles);

	// Stuff
	STATIC_CAST(CEikChoiceList*,Control(EDlgStuffREUSize))->SetCurrentItem(iPrefs->REUSize);
	SetChoiceListCurrentItem(EDlgStuffFastReset, iPrefs->FastReset);

#elif defined (__SERIES60__)
	/// @todo placeholder
#elif defined (__QUARTZ__)
	/// @todo placeholder
#else

	// 1541
	STATIC_CAST(CEikCheckBox*,Control(EDlg1541Emul1541Proc))->SetState((CEikButtonBase::TState)iPrefs->Emul1541Proc);

	// Joystick
	STATIC_CAST(CEikCheckBox*,Control(EDlgJoystick1On))->SetState((CEikButtonBase::TState)iPrefs->Joystick1On);
	STATIC_CAST(CEikCheckBox*,Control(EDlgJoystick2On))->SetState((CEikButtonBase::TState)iPrefs->Joystick2On);

	// VIC
	STATIC_CAST(CEikNumberEditor*,Control(EDlgVICSkipFrames))->SetNumber(iPrefs->SkipFrames);
	STATIC_CAST(CEikCheckBox*,Control(EDlgVICSpritesOn))->SetState((CEikButtonBase::TState)iPrefs->SpritesOn);
	STATIC_CAST(CEikCheckBox*,Control(EDlgVICSpriteColl))->SetState((CEikButtonBase::TState)iPrefs->SpriteCollisions);

	// SID
	STATIC_CAST(CEikChoiceList*,Control(EDlgSIDType))->SetCurrentItem(iPrefs->SIDType);
	STATIC_CAST(CEikCheckBox*,Control(EDlgSIDFilters))->SetState((CEikButtonBase::TState)iPrefs->SIDFilters);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMin))->SetNumber(iPrefs->LatencyMin);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMax))->SetNumber(iPrefs->LatencyMax);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyAvg))->SetNumber(iPrefs->LatencyAvg);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDVolume))->SetNumber(iPrefs->iVolume);
	
	// CPU
	STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUNormalCycles))->SetNumber(iPrefs->NormalCycles);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUBadLineCycles))->SetNumber(iPrefs->BadLineCycles);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUCIACycles))->SetNumber(iPrefs->CIACycles);
	STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUFloppyCycles))->SetNumber(iPrefs->FloppyCycles);

	// Stuff
	STATIC_CAST(CEikChoiceList*,Control(EDlgStuffREUSize))->SetCurrentItem(iPrefs->REUSize);
	STATIC_CAST(CEikCheckBox*,Control(EDlgStuffFastReset))->SetState((CEikButtonBase::TState)iPrefs->FastReset);
	
	MakePanelButtonVisible(EEikBidSpace, EFalse);
#endif
	}


void CPrefsDialog::SetInitialCurrentLine()
/**
 * virtual method called by CEikDialog framework
 */
	{

//
// From Jal:
// I removed the SetActivePageByIdL from the PreLayoutDynInitL and put it in
// the SetInitialCurrentLine instead of the DrawNow.
//

#if defined(__CRYSTAL__)
	SetActivePageByIdL(EDlgFocusJoystick);
#elif defined(__SERIES60__)
	/// @todo placeholder
#else
	TRAPD(ret, TryChangeFocusToL(EDlgFocusJoystick));
	ELOG(if(ret != KErrNone))
	  ELOG2(_L8("WARNING: could not change focus [ret=%d]"), ret );
#endif
	}


_LIT8(KD64, "D64");
_LIT8(KT64, "T64");


TBool CPrefsDialog::OkToExitL(TInt aControlId)
/**
 * called when about to leave the dialog box
 */
	{
	TInt focusItemId = IdOfFocusControl();

	if (aControlId==EButGetFile)
		{
		TFileName diskImagePathAndName;
		CEikEdwin* edwin = STATIC_CAST(CEikEdwin*,Control(focusItemId));
		edwin->GetText(diskImagePathAndName); 

		if(diskImagePathAndName.Length() == 0)
			diskImagePathAndName.Copy(_L("C:\\"));
		
		TBool success = EFalse;

		//
		// the only difference here between ER5 and ER6 is the
		// concrete file open controls
		//
#if defined(__CRYSTAL__)
		success = CCknOpenFileDialog::RunDlgLD(diskImagePathAndName);
#elif defined(__SERIES60__)
		/// @todo placeholder
#elif defined(__QUARTZ__)
		/// @todo placeholder
#else // __ER6__
		CDirContentsListBoxModel::TSortOrder order = CDirContentsListBoxModel::EOrderByName;
		CEikFileBrowserDialog* browseDialog = new (ELeave) 
		  CEikFileBrowserDialog(diskImagePathAndName, CEikFileBrowser::EShowSystem, order);
		success = browseDialog->ExecuteLD(R_EIK_DIALOG_FILE_BROWSE);
#endif
		if(success)
			edwin->SetTextL(&diskImagePathAndName);

	    return(EFalse);
        }

	if (aControlId==EEikBidOk)
		{

		TBool joy_reminder = EFalse;
#if !defined(__ER6__)
		/**
		 * remind the user to turn on the capslock if joystick is enabled (only once)
		 */
		RWsSession ws;
		const TInt ret = ws.Connect();
		if(!ret && !(EModifierCapsLock & ws.GetModifierState()) )
			{

			if(
			  (!iPrefs->Joystick1On && ChoiceListCurrentItem(EDlgJoystick1On))
			  ||
			  (!iPrefs->Joystick2On && ChoiceListCurrentItem(EDlgJoystick2On))
			  )
				joy_reminder = ETrue;
			if(
			  (!iPrefs->Joystick1On && ((CEikCheckBox*)Control(EDlgJoystick1On))->State())
			  ||
			  (!iPrefs->Joystick2On && ((CEikCheckBox*)Control(EDlgJoystick2On))->State())
			  )
				joy_reminder = ETrue;
			
			ws.Close();
			}
#endif


		// 1541

#if defined(__CRYSTAL__)
		iPrefs->Emul1541Proc = ChoiceListCurrentItem(EDlg1541Emul1541Proc);
#elif defined(__SERIES60__)
		/// @todo placeholder
#elif defined(__QUARTZ__)
		/// @todo placeholder
#else // __ER6__
		iPrefs->Emul1541Proc = STATIC_CAST(CEikCheckBox*,Control(EDlg1541Emul1541Proc))->State();
#endif

		for(TInt i=0;i<4;i++)
			{
			TText8* buf= (TText8*)iPrefs->DrivePath[i];
			TInt bufLen = TPtrC8(buf).Length();
			TInt maxBufLen = sizeof(iPrefs->DrivePath[i]);
			TPtr8 drivePath(buf,bufLen,maxBufLen);
            TFileName tmp;
			STATIC_CAST(CEikEdwin*,Control(EDlg1541Drive8+i))->GetText(tmp);
            drivePath.Copy(tmp);
			drivePath.ZeroTerminate();

			if(drivePath.Length() == 0)
				{
				iPrefs->DriveType[i]=DRVTYPE_DIR;
				}
			else
				if (drivePath.Right(3).CompareF(KD64)==0)
					iPrefs->DriveType[i]=DRVTYPE_D64;
				else if (drivePath.Right(3).CompareF(KT64)==0)
					iPrefs->DriveType[i]=DRVTYPE_T64;
				else
					iPrefs->DriveType[i]=DRVTYPE_DIR;
			}

#if defined(__CRYSTAL__)

		// Joystick
		iPrefs->Joystick1On = ChoiceListCurrentItem(EDlgJoystick1On);
		iPrefs->Joystick2On = ChoiceListCurrentItem(EDlgJoystick2On);

		// VIC
		iPrefs->SkipFrames = NumberEditorValue(EDlgVICSkipFrames);
		iPrefs->SpritesOn = ChoiceListCurrentItem(EDlgVICSpritesOn);
		iPrefs->SpriteCollisions = ChoiceListCurrentItem(EDlgVICSpriteColl);

		/** @todo SID (not yet!)
		iPrefs->SIDType = STATIC_CAST(CEikChoiceList*,Control(EDlgSIDType))->CurrentItem();
		iPrefs->SIDFilters = STATIC_CAST(CEikCheckBox*,Control(EDlgSIDFilters))->State();
		iPrefs->LatencyMin = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMin))->Number();
		iPrefs->LatencyMax = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMax))->Number();
		iPrefs->LatencyAvg = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyAvg))->Number();
		iPrefs->iVolume = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDVolume))->Number();
		**/

		// CPU
		iPrefs->NormalCycles = NumberEditorValue(EDlgCPUNormalCycles);
		iPrefs->BadLineCycles = NumberEditorValue(EDlgCPUBadLineCycles);
		iPrefs->CIACycles = NumberEditorValue(EDlgCPUCIACycles);
		iPrefs->FloppyCycles = NumberEditorValue(EDlgCPUFloppyCycles);

		// Stuff
		iPrefs->REUSize = ChoiceListCurrentItem(EDlgStuffREUSize);
		iPrefs->FastReset = ChoiceListCurrentItem(EDlgStuffFastReset);

#elif defined(__SERIES60__)
		/// @todo placeholder
#elif defined(__QUARTZ__)
		/// @todo placeholder
#else // __ER6__

		// Joystick
		iPrefs->Joystick1On = STATIC_CAST(CEikCheckBox*,Control(EDlgJoystick1On))->State();
		iPrefs->Joystick2On = STATIC_CAST(CEikCheckBox*,Control(EDlgJoystick2On))->State();

		// VIC
		iPrefs->SkipFrames = STATIC_CAST(CEikNumberEditor*,Control(EDlgVICSkipFrames))->Number();
		iPrefs->SpritesOn = STATIC_CAST(CEikCheckBox*,Control(EDlgVICSpritesOn))->State();
		iPrefs->SpriteCollisions = STATIC_CAST(CEikCheckBox*,Control(EDlgVICSpriteColl))->State();

		// SID
		iPrefs->SIDType = STATIC_CAST(CEikChoiceList*,Control(EDlgSIDType))->CurrentItem();
		iPrefs->SIDFilters = STATIC_CAST(CEikCheckBox*,Control(EDlgSIDFilters))->State();
		iPrefs->LatencyMin = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMin))->Number();
		iPrefs->LatencyMax = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyMax))->Number();
		iPrefs->LatencyAvg = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDLatencyAvg))->Number();
		iPrefs->iVolume = STATIC_CAST(CEikNumberEditor*,Control(EDlgSIDVolume))->Number();

		// CPU
		iPrefs->NormalCycles = STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUNormalCycles))->Number();
		iPrefs->BadLineCycles = STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUBadLineCycles))->Number();
		iPrefs->CIACycles = STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUCIACycles))->Number();
		iPrefs->FloppyCycles = STATIC_CAST(CEikNumberEditor*,Control(EDlgCPUFloppyCycles))->Number();

		// Stuff
		iPrefs->REUSize = STATIC_CAST(CEikChoiceList*,Control(EDlgStuffREUSize))->CurrentItem();
		iPrefs->FastReset = STATIC_CAST(CEikCheckBox*,Control(EDlgStuffFastReset))->State();

#endif // __ER6__


		// if no changes, do no changes
		if(*iPrefs == *iOrigPrefs)
			return ETrue;			// ETrue means close Dialog box

		// Pause Emulator if running
		TBool paused = iC64->Paused();

		if(!paused)
			iC64->TogglePause();

		iC64->NewPrefs(iPrefs);

		*iOrigPrefs = *iPrefs;

		if(!paused)
			iC64->TogglePause();

		_LIT(KPrefsChange, "prefs changed");
		_LIT(KJoyOnText, "enable capslock to use joystick");

		if(joy_reminder)
			iEikonEnv->InfoMsg(KJoyOnText);
		else
			iEikonEnv->InfoMsg(KPrefsChange);

		return ETrue;
		}
	
#ifdef __CRYSTAL__
	if (focusItemId > 0)
		{
		TInt nextFocusItemId=ChoiceListCurrentItem(focusItemId);
		TryChangeFocusToL(1+nextFocusItemId);
	    return EFalse;
		}
#endif

	return EFalse;
	}


void CPrefsDialog::PageChangedL(TInt aControlId)
/**
 * This function is called by the framework when when the user selects a
 * different page in a multi-page dialog.
 */
	{
#if defined(__ER6__)
	(void)aControlId;
	CEikButtonGroupContainer& cbaToolbar = ButtonGroupContainer();
	const TBool wasDimmed = cbaToolbar.IsCommandDimmed(EButGetFile);
	cbaToolbar.DimCommand(EButGetFile, ETrue);
	// Redraw if changed
	if (wasDimmed != cbaToolbar.IsCommandDimmed(EButGetFile))
		cbaToolbar.ButtonById(EButGetFile)->DrawNow();
#else // __ER6__
	const TBool visible = (aControlId == EDlgFocus1541);
	MakePanelButtonVisible(EButGetFile, visible);
#endif
	}


#if defined(__ER6__)

void CPrefsDialog::LineChangedL(TInt aControlId) 
/**
 * This function is called by the framework when when the user selects a
 * different line in a dialog.
 */
	{
	CEikButtonGroupContainer& cbaToolbar = ButtonGroupContainer();
    
	const TBool wasDimmed = cbaToolbar.IsCommandDimmed(EButGetFile);
	cbaToolbar.DimCommand(EButGetFile, ETrue);
	if (ActivePageId() == EDlgFocus1541)
		if (aControlId == EDlg1541Drive8 || aControlId == EDlg1541Drive9 
			|| aControlId == EDlg1541Drive10 || aControlId == EDlg1541Drive11)
			cbaToolbar.DimCommand(EButGetFile, EFalse);
	// Redraw if changed
	if (wasDimmed != cbaToolbar.IsCommandDimmed(EButGetFile))
		cbaToolbar.ButtonById(EButGetFile)->DrawNow();

    }
#endif // __ER6__



// EOF - PREFSDIALOG.CPP
