// recfrodo.cpp
//
// Copyright (c) 2000-2001 Alfred E. Heggestad
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License version 2 as
//    published by the Free Software Foundation.
//

/**
 * @file recfrodo.cpp
 *
 * Implements the class CRecfrodo for recognizing Frodo files
 */

#include "recfrodo.h"

#ifdef _UNICODE
const TUid KUidRecFrodo = {0x10005f47};
#else
const TUid KUidRecFrodo = {0x10005f46};
#endif

const TInt KMinBufferLength = 13;  ///< minimum amount of file needed to determine a Frodo snapshot file
const TInt KMaxBufferLength = 64;  ///< maximum amount of buffer space we will ever use

_LIT8(KRecFrodoSignature, "FrodoSnapshot");
_LIT8(KDataTypeFrodo, "application/frodo"); /// @todo is this correct ?


//
// implementation of CRecFrodo
//


CRecFrodo::CRecFrodo()
/**
 * C'tor
 */
	:CApaDataRecognizerType(KUidRecFrodo, CApaDataRecognizerType::ELow)
	{
	iCountDataTypes = 1;
	}


TUint CRecFrodo::PreferredBufSize()
/**
 * returns the preferred buffer size
 */
	{
	return KMaxBufferLength;
	}


TDataType CRecFrodo::SupportedDataTypeL(TInt aIndex) const
/**
 * returns the supported data types
 */
	{
	__ASSERT_DEBUG(aIndex==0, User::Invariant());
	return TDataType(KDataTypeFrodo);
	}


void CRecFrodo::DoRecognizeL(const TDesC& /* aName */, const TDesC8& aBuffer)
/**
 * called by the Application Architecture Framework.
 *
 */
	{
	// check if file bigger than 5 bytes
	if (aBuffer.Length() < KMinBufferLength)
		return;
  
	// search for Frodo snapshot header
	if (aBuffer.FindF(KRecFrodoSignature) >= 0)
		{
		// Yes it was found
		iConfidence = ECertain;
		iDataType = TDataType(KDataTypeFrodo);
		}
	}
 

EXPORT_C CApaDataRecognizerType* CreateRecognizer()
/**
 * The gate function - ordinal 1
 *
 * Create a new Frodo recogniser object (CRecFrodo) and return
 * the pointer to it.
 *
 * @return pointer to a newly created CRecFrodo object
 * @retval NULL out-of memory, no object created
 */
	{
	return new CRecFrodo();
	}


GLDEF_C TInt E32Dll(TDllReason /*aReason*/)
/**
 * DLL entry point
 */
	{
	return KErrNone;
	}

// EOF - recfrodo.cpp
