// recfrodofile.cpp
//
// Copyright (c) 2000-2002 Alfred E. Heggestad
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License version 2 as
//    published by the Free Software Foundation.
//

/**
 * @file recfrodofile.cpp
 *
 * Implements the class CRecfrodofile for recognizing Frodo files
 */

#include "recfrodofile.h"
#include <e32svr.h>
#include <s32file.h>
#include <f32file.h>
#include <apaid.h>
#include <apacmdln.h>
#include <apgcli.h>


/**
 * This UID specifies the UID of the Frodo application
 * which will started when user clicks on a Frodo file
 * These UIDs are defined in e32frodo.mmp
 */
#ifdef _UNICODE
const TUid KUidRecFrodo = {0x10005f47};
#else
const TUid KUidRecFrodo = {0x10005f46};
#endif

_LIT8(KRecFrodoSignature, "FrodoSnapshot");


CApaFileRecognizerType::TRecognizedType CRecFrodoFile::DoRecognizeFileL(RFs& aFs, TUidType /*aType*/)
/**
 * Recognises Frodo files - by examining the file's contents
 */
	{
	iRecognizedType = ENotRecognized;

	// Frodo snapshot files are recognized by contents
	RFile file;
	if(file.Open(aFs, *iFullFileName, EFileShareReadersOnly) != KErrNone)
		return iRecognizedType;
	TBuf8<1024> buf;
	TInt err = file.Read(buf);
	file.Close();
	if (err!=KErrNone)
		return iRecognizedType;
	if (buf.FindF(KRecFrodoSignature) != KErrNotFound)
		{
		iRecognizedType = EDoc;
		iAppUid = KUidRecFrodo;
		return iRecognizedType;
		}
	return iRecognizedType;
	}


TThreadId CRecFrodoFile::RunL(TApaCommand aCommand, const TDesC* aDocFileName, const TDesC8* aTailEnd) const
	{
	CApaCommandLine* commandLine = CApaCommandLine::NewLC();
	commandLine->SetCommandL(aCommand);
	if(iRecognizedType == EDoc)
		{
		// We've recognised a doc so we need to scan for its associated app
		TApaAppEntry appEntry;
		User::LeaveIfError(iFileRecognizer->AppLocator()->GetAppEntryByUid(appEntry,iAppUid));
		commandLine->SetLibraryNameL(appEntry.iFullName);
		if(aDocFileName)
			commandLine->SetDocumentNameL(*iFullFileName);
		}
	else
		User::Leave(KErrNotSupported);

	TThreadId id = AppRunL(*commandLine);
	CleanupStack::PopAndDestroy(); // commandLine
	return id;
	}


EXPORT_C CApaFileRecognizerType* CreateRecognizer()
/**
 * Create a new Frodo recogniser object and return
 * the pointer to it. Called by AppArc
 *
 * @return poiner to newly created CRecFrodoFile object
 * @retval NULL out-of memory, no object created
 */
	{
	return new CRecFrodoFile();
	}


GLDEF_C TInt E32Dll(TDllReason /*aReason*/)
/**
 * DLL entry point
 */
	{
	return KErrNone;
	}


// EOF - recfrodofile.cpp
