/*
 *  Prefs.cpp - Global preferences
 *
 *  Frodo (C) 1994-1997 Christian Bauer
 */

#include "sysdeps.h"

#include "Prefs.h"
#include "Display.h"
#include "C64.h"
#include "main.h"


#ifndef __SYMBIAN32__
// These are the active preferences
Prefs ThePrefs;

// These are the preferences on disk
Prefs ThePrefsOnDisk;
#endif // !__SYMBIAN32__


// 
// The default color mappings are defined here (C64 -> Epoc)
//

#if COLOR_DEPTH == 4
const uint8 colorRefTable[16] = 
	{
				// --C64--
		0x00,	// black
		0x0f,	// white
		0x02,	// dark red
		0x0a,	// cyan
		0x08,	// purple
		0x04,	// dark green
		0x0c,	// dark blue
		0x06,	// yellow
		0x03,	// light brown
		0x0b,	// dark brown
		0x05,	// light red
		0x01,	// dark grey
		0x0d,	// med grey
		0x07,	// light green
		0x09,	// light blue
		0x0e	// light grey
	};
#elif COLOR_DEPTH == 8

//
// ER6 means Crystal and 12 bits color depth.
//
#ifdef __ER6__
const uint8 colorRefTable[16] = 
	{
			// --C64--
		0,	// black
		15,	// white
		2,	// dark red
		10,	// cyan
		8,	// purple	
		4,	// dark green
		12,	// dark blue
		6,	// yellow
		3,	// brown
		11,	// dark brown
		5,	// light red
		1,	// dark grey
		13,	// med grey
		7,	// light green
		9,	// light blue
		14	// light grey
	};
#else
const uint8 colorRefTable[16] = 
	{
				// --C64--
		0x00,	// black
		0xff,	// white
		0x03,	// dark red
		0xd2,	// cyan
		0xc8,	// purple	
		0x19,	// dark green
		0x82,	// dark blue
		0x23,	// yellow
		0x0f,	// brown
		0x12,	// dark brown
		0x05,	// light red
		0x54,	// dark grey
		0x55,	// med grey
		0x1e,	// light green
		0xf0,	// light blue
		0xff	// light grey
	};
#endif

#else
ERROR!
#endif	// COLOR_DEPTH




/*
 *  Constructor: Set up preferences with defaults
 */

Prefs::Prefs()
{

	// 1541
	Emul1541Proc = false;
	for (int i=0; i<4; i++)
		DriveType[i] = DRVTYPE_DIR;

	strcpy(DrivePath[0], "c:\\c64");
	strcpy(DrivePath[1], "");
	strcpy(DrivePath[2], "");
	strcpy(DrivePath[3], "");

	// Joystick
	Joystick1On = false;
	Joystick2On = false;

	// VIC
	SkipFrames = 2;
	SpritesOn = true;
	SpriteCollisions = true;

	// SID
	SIDType = SIDTYPE_DIGITAL;
	SIDFilters = true;
	LatencyMin = 80;
	LatencyMax = 120;
	LatencyAvg = 280;

	// CPU
	NormalCycles = 63;
	BadLineCycles = 23;
	CIACycles = 63;
	FloppyCycles = 64;

	// Stuff
	REUSize = REU_NONE;
	LimitSpeed = false;
	FastReset = false;
	iShowToolbar = true;

	// Colors
	for(int j=0;j<16;j++)
		ColorTbl[j] = colorRefTable[j];

#ifdef __SYMBIAN32__

	// offset
	TSize screensize = TSize(0, 200);
	TMachineInfoV1Buf machine;
	const TInt ret = UserHal::MachineInfo(machine);
	if(ret == KErrNone)
		{
		screensize = machine().iDisplaySizeInPixels;
		}

	iOffset.SetXY(0, ((screensize.iHeight-DISPLAY_Y) / 2) +1);
	iVolume = 7; /// @todo determine max volume
	
#else // __SYMBIAN32__

	CIAIRQHack = false;
	MapSlash = true;
	JoystickSwap = false;
	ScalingNumerator = 2;
	ScalingDenominator = 2;
	DisplayType = DISPTYPE_WINDOW;
	strcpy(ViewPort, "Default");
	strcpy(DisplayMode, "Default");
	DoubleScan = true;
	HideCursor = false;
	DirectSound = true;	
	ExclusiveSound = false;
	AutoPause = false;
	PrefsAtStartup = false;
	SystemMemory = false;
	AlwaysCopy = false;
	SystemKeys = true;
	ShowLEDs = true;

#endif // __SYMBIAN32__

	CTOR(Prefs);

} // end, Prefs::Prefs


Prefs::~Prefs()
/**
 *  Destructor
 */
	{
	DTOR(Prefs);
	}


bool Prefs::operator==(const Prefs &rhs) const
/**
 *  Check if two Prefs structures are equal
 */
{
	return (1
		&& NormalCycles == rhs.NormalCycles
		&& BadLineCycles == rhs.BadLineCycles
		&& CIACycles == rhs.CIACycles
		&& FloppyCycles == rhs.FloppyCycles
		&& SkipFrames == rhs.SkipFrames
		&& LatencyMin == rhs.LatencyMin
		&& LatencyMax == rhs.LatencyMax
		&& LatencyAvg == rhs.LatencyAvg
		&& DriveType[0] == rhs.DriveType[0]
		&& DriveType[1] == rhs.DriveType[1]
		&& DriveType[2] == rhs.DriveType[2]
		&& DriveType[3] == rhs.DriveType[3]
		&& strcmp(DrivePath[0], rhs.DrivePath[0]) == 0
		&& strcmp(DrivePath[1], rhs.DrivePath[1]) == 0
		&& strcmp(DrivePath[2], rhs.DrivePath[2]) == 0
		&& strcmp(DrivePath[3], rhs.DrivePath[3]) == 0
		&& SIDType == rhs.SIDType
		&& REUSize == rhs.REUSize
		&& SpritesOn == rhs.SpritesOn
		&& SpriteCollisions == rhs.SpriteCollisions
		&& Joystick1On == rhs.Joystick1On
		&& Joystick2On == rhs.Joystick2On
		&& LimitSpeed == rhs.LimitSpeed
		&& FastReset == rhs.FastReset
		&& Emul1541Proc == rhs.Emul1541Proc
		&& SIDFilters == rhs.SIDFilters

#ifdef __SYMBIAN32__

		&& iShowToolbar == rhs.iShowToolbar
		&& !Mem::Compare(ColorTbl, 16, rhs.ColorTbl, 16)	// colors
		&& iOffset == rhs.iOffset // XY offset
		&& iVolume == rhs.iVolume // XY offset

#else // __SYMBIAN32__

    	&& CIAIRQHack == rhs.CIAIRQHack
		&& MapSlash == rhs.MapSlash
		&& ScalingNumerator == rhs.ScalingNumerator
		&& ScalingDenominator == rhs.ScalingNumerator
		&& strcmp(ViewPort, rhs.ViewPort) == 0
		&& strcmp(DisplayMode, rhs.DisplayMode) == 0
		&& DisplayType == rhs.DisplayType
		&& JoystickSwap == rhs.JoystickSwap
		&& DoubleScan == rhs.DoubleScan
		&& HideCursor == rhs.HideCursor
		&& DirectSound == rhs.DirectSound
		&& ExclusiveSound == rhs.ExclusiveSound
		&& AutoPause == rhs.AutoPause
		&& PrefsAtStartup == rhs.PrefsAtStartup
		&& SystemMemory == rhs.SystemMemory
		&& AlwaysCopy == rhs.AlwaysCopy
		&& SystemKeys == rhs.SystemKeys
		&& ShowLEDs == rhs.ShowLEDs

#endif // !__SYMBIAN32__

	);
}

bool Prefs::operator!=(const Prefs &rhs) const
{
	return !operator==(rhs);
}


/*
 *  Check preferences for validity and correct if necessary
 */

void Prefs::Check(void)
{
	if (SkipFrames <= 0) SkipFrames = 1;

	if (SIDType < SIDTYPE_NONE || SIDType > SIDTYPE_SIDCARD)
		SIDType = SIDTYPE_NONE;

	if (REUSize < REU_NONE || REUSize > REU_512K)
		REUSize = REU_NONE;

#ifndef __SYMBIAN32__
	if (DisplayType < DISPTYPE_WINDOW || DisplayType > DISPTYPE_SCREEN)
		DisplayType = DISPTYPE_WINDOW;
#endif

	for (int i=0; i<4; i++)
		if (DriveType[i] < DRVTYPE_DIR || DriveType[i] > DRVTYPE_T64)
			DriveType[i] = DRVTYPE_DIR;
}


/*
 *  Load preferences from file
 */

bool Prefs::Load(char *filename)
{
	FILE *file;
	char line[256], keyword[256], value[256];

	if ((file = fopen(filename, "r")) != NULL) {
		while(fgets(line, 255, file)) {
			if (sscanf(line, "%s = %s\n", keyword, value) == 2) 
				{

				// 1541
				if (!strcmp(keyword, "Emul1541Proc"))
					Emul1541Proc = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "DriveType8"))
					if (!strcmp(value, "DIR"))
						DriveType[0] = DRVTYPE_DIR;
					else if (!strcmp(value, "D64"))
						DriveType[0] = DRVTYPE_D64;
					else
						DriveType[0] = DRVTYPE_T64;
				else if (!strcmp(keyword, "DriveType9"))
					if (!strcmp(value, "DIR"))
						DriveType[1] = DRVTYPE_DIR;
					else if (!strcmp(value, "D64"))
						DriveType[1] = DRVTYPE_D64;
					else
						DriveType[1] = DRVTYPE_T64;
				else if (!strcmp(keyword, "DriveType10"))
					if (!strcmp(value, "DIR"))
						DriveType[2] = DRVTYPE_DIR;
					else if (!strcmp(value, "D64"))
						DriveType[2] = DRVTYPE_D64;
					else
						DriveType[2] = DRVTYPE_T64;
				else if (!strcmp(keyword, "DriveType11"))
					if (!strcmp(value, "DIR"))
						DriveType[3] = DRVTYPE_DIR;
					else if (!strcmp(value, "D64"))
						DriveType[3] = DRVTYPE_D64;
					else
						DriveType[3] = DRVTYPE_T64;
				else if (!strcmp(keyword, "DrivePath8"))
					strcpy(DrivePath[0], value);
				else if (!strcmp(keyword, "DrivePath9"))
					strcpy(DrivePath[1], value);
				else if (!strcmp(keyword, "DrivePath10"))
					strcpy(DrivePath[2], value);
				else if (!strcmp(keyword, "DrivePath11"))
					strcpy(DrivePath[3], value);

				// Joystick
				else if (!strcmp(keyword, "Joystick1On"))
					Joystick1On = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "Joystick2On"))
					Joystick2On = !strcmp(value, "TRUE");

				// VIC
				else if (!strcmp(keyword, "SkipFrames"))
					SkipFrames = atoi(value);
				else if (!strcmp(keyword, "SpritesOn"))
					SpritesOn = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "SpriteCollisions"))
					SpriteCollisions = !strcmp(value, "TRUE");

				// SID
				else if (!strcmp(keyword, "SIDType"))
					if (!strcmp(value, "DIGITAL"))
						SIDType = SIDTYPE_DIGITAL;
					else if (!strcmp(value, "SIDCARD"))
						SIDType = SIDTYPE_SIDCARD;
					else
						SIDType = SIDTYPE_NONE;
				else if (!strcmp(keyword, "SIDFilters"))
					SIDFilters = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "LatencyMin"))
					LatencyMin = atoi(value);
				else if (!strcmp(keyword, "LatencyMax"))
					LatencyMax = atoi(value);
				else if (!strcmp(keyword, "LatencyAvg"))
					LatencyAvg = atoi(value);

				// CPU
				else if (!strcmp(keyword, "NormalCycles"))
					NormalCycles = atoi(value);
				else if (!strcmp(keyword, "BadLineCycles"))
					BadLineCycles = atoi(value);
				else if (!strcmp(keyword, "CIACycles"))
					CIACycles = atoi(value);
				else if (!strcmp(keyword, "FloppyCycles"))
					FloppyCycles = atoi(value);

				// Stuff
				else if (!strcmp(keyword, "REUSize")) {
					if (!strcmp(value, "128K"))
						REUSize = REU_128K;
					else if (!strcmp(value, "256K"))
						REUSize = REU_256K;
					else if (!strcmp(value, "512K"))
						REUSize = REU_512K;
					else
						REUSize = REU_NONE;
				} 
				else if (!strcmp(keyword, "LimitSpeed"))
					LimitSpeed = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "FastReset"))
					FastReset = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "ShowToolbar"))
					iShowToolbar = !strcmp(value, "TRUE");

				// Colors
				else if (!strcmp(keyword, "Color0Black"))
					ColorTbl[0] = atoi(value);
				else if (!strcmp(keyword, "Color1White"))
					ColorTbl[1] = atoi(value);
				else if (!strcmp(keyword, "Color2DarkRed"))
					ColorTbl[2] = atoi(value);
				else if (!strcmp(keyword, "Color3Cyan"))
					ColorTbl[3] = atoi(value);
				else if (!strcmp(keyword, "Color4Purple"))
					ColorTbl[4] = atoi(value);
				else if (!strcmp(keyword, "Color5DarkGreen"))
					ColorTbl[5] = atoi(value);
				else if (!strcmp(keyword, "Color6DarkBlue"))
					ColorTbl[6] = atoi(value);
				else if (!strcmp(keyword, "Color7Yellow"))
					ColorTbl[7] = atoi(value);
				else if (!strcmp(keyword, "Color8LightBrown"))
					ColorTbl[8] = atoi(value);
				else if (!strcmp(keyword, "Color9DarkBrown"))
					ColorTbl[9] = atoi(value);
				else if (!strcmp(keyword, "Color10LightRed"))
					ColorTbl[10] = atoi(value);
				else if (!strcmp(keyword, "Color11DarkGrey"))
					ColorTbl[11] = atoi(value);
				else if (!strcmp(keyword, "Color12MediumGrey"))
					ColorTbl[12] = atoi(value);
				else if (!strcmp(keyword, "Color13LightGreen"))
					ColorTbl[13] = atoi(value);
				else if (!strcmp(keyword, "Color14LightBlue"))
					ColorTbl[14] = atoi(value);
				else if (!strcmp(keyword, "Color15LightGrey"))
					ColorTbl[15] = atoi(value);

#ifdef __SYMBIAN32__

				// XY offset
				else if (!strcmp(keyword, "Offset_X"))
					iOffset.iX = atoi(value);
				else if (!strcmp(keyword, "Offset_Y"))
					iOffset.iY = atoi(value);
				else if (!strcmp(keyword, "Volume"))
					iVolume = atoi(value);

#else // __SYMBIAN32__

				else if (!strcmp(keyword, "CIAIRQHack"))
					CIAIRQHack = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "MapSlash"))
					MapSlash = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "ScalingNumerator"))
					ScalingNumerator = atoi(value);
				else if (!strcmp(keyword, "ScalingDenominator"))
					ScalingDenominator = atoi(value);
				else if (!strcmp(keyword, "ViewPort"))
					strcpy(ViewPort, value);
				else if (!strcmp(keyword, "DisplayMode"))
					strcpy(DisplayMode, value);
				else if (!strcmp(keyword, "DisplayType"))
					DisplayType = strcmp(value, "SCREEN") ? DISPTYPE_WINDOW : DISPTYPE_SCREEN;
				else if (!strcmp(keyword, "JoystickSwap"))
					JoystickSwap = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "DoubleScan"))
					DoubleScan = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "HideCursor"))
					HideCursor = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "DirectSound"))
					DirectSound = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "ExclusiveSound"))
					ExclusiveSound = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "AutoPause"))
					AutoPause = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "PrefsAtStartup"))
					PrefsAtStartup = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "SystemMemory"))
					SystemMemory = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "AlwaysCopy"))
					AlwaysCopy = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "SystemKeys"))
					SystemKeys = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "ShowLEDs"))
					ShowLEDs = !strcmp(value, "TRUE");

#endif // !__SYMBIAN32__

			}
		}
		fclose(file);
	}
	else 
	{
	ELOG1(_L8("Warning: Prefs not loaded!\n"));
	return false;
	}

	Check();
	ELOG2(_L8("Prefs loaded from '%s'\n"), filename);
	return true;

#ifndef __SYMBIAN32__
	//	ThePrefsOnDisk = *this;	//TODO: use it
#endif
}


/*
 *  Save preferences to file
 *  true: success, false: error
 */

bool Prefs::Save(char *filename)
{
	FILE *file;

	Check();
	if ((file = fopen(filename, "w")) != NULL) 
		{
		fprintf(file, "// Prefs for EPOC32 Frodo: \n" );
		fprintf(file, "// ----------------------- \n" );

		// 1541
		fprintf(file, "// 1541:\n" );
		fprintf(file, "Emul1541Proc = %s\n", Emul1541Proc ? "TRUE" : "FALSE");
		for (int i=0; i<4; i++) {
			fprintf(file, "DriveType%d = ", i+8);
			switch (DriveType[i]) {
				case DRVTYPE_DIR:
					fprintf(file, "DIR\n");
					break;
				case DRVTYPE_D64:
					fprintf(file, "D64\n");
					break;
				case DRVTYPE_T64:
					fprintf(file, "T64\n");
					break;
			}
			fprintf(file, "DrivePath%d = %s\n", i+8, DrivePath[i]);
		}

		// Joystick
		fprintf(file, "// Joystick:\n" );
		fprintf(file, "Joystick1On = %s\n", Joystick1On ? "TRUE" : "FALSE");
		fprintf(file, "Joystick2On = %s\n", Joystick2On ? "TRUE" : "FALSE");

		// VIC
		fprintf(file, "// VIC:\n" );
		fprintf(file, "SkipFrames = %d\n", SkipFrames);
		fprintf(file, "SpritesOn = %s\n", SpritesOn ? "TRUE" : "FALSE");
		fprintf(file, "SpriteCollisions = %s\n", SpriteCollisions ? "TRUE" : "FALSE");

		// SID
		fprintf(file, "// SID:\n" );
		fprintf(file, "SIDType = ");
		switch (SIDType) {
			case SIDTYPE_NONE:
				fprintf(file, "NONE\n");
				break;
			case SIDTYPE_DIGITAL:
				fprintf(file, "DIGITAL\n");
				break;
			case SIDTYPE_SIDCARD:
				fprintf(file, "SIDCARD\n");
				break;
		}
		fprintf(file, "SIDFilters = %s\n", SIDFilters ? "TRUE" : "FALSE");
		fprintf(file, "LatencyMin = %d\n", LatencyMin);
		fprintf(file, "LatencyMax = %d\n", LatencyMax);
		fprintf(file, "LatencyAvg = %d\n", LatencyAvg);

		// CPU
		fprintf(file, "// CPU:\n" );
		fprintf(file, "NormalCycles = %d\n", NormalCycles);
		fprintf(file, "BadLineCycles = %d\n", BadLineCycles);
		fprintf(file, "CIACycles = %d\n", CIACycles);
		fprintf(file, "FloppyCycles = %d\n", FloppyCycles);

		// Stuff
		fprintf(file, "// Stuff:\n" );
		fprintf(file, "REUSize = ");
		switch (REUSize) {
			case REU_NONE:
				fprintf(file, "NONE\n");
				break;
			case REU_128K:
				fprintf(file, "128K\n");
				break;
			case REU_256K:
				fprintf(file, "256K\n");
				break;
			case REU_512K:
				fprintf(file, "512K\n");
				break;
		};
		fprintf(file, "LimitSpeed = %s\n", LimitSpeed ? "TRUE" : "FALSE");
		fprintf(file, "FastReset = %s\n", FastReset ? "TRUE" : "FALSE");
		fprintf(file, "ShowToolbar = %s\n", iShowToolbar ? "TRUE" : "FALSE");

		// Colors
		fprintf(file, "// Colors:\n" );
		fprintf(file, "Color0Black = %d\n",       ColorTbl[0]);
		fprintf(file, "Color1White = %d\n",       ColorTbl[1]);
		fprintf(file, "Color2DarkRed = %d\n",     ColorTbl[2]);
		fprintf(file, "Color3Cyan = %d\n",        ColorTbl[3]);
		fprintf(file, "Color4Purple = %d\n",      ColorTbl[4]);
		fprintf(file, "Color5DarkGreen = %d\n",   ColorTbl[5]);
		fprintf(file, "Color6DarkBlue = %d\n",    ColorTbl[6]);
		fprintf(file, "Color7Yellow = %d\n",      ColorTbl[7]);
		fprintf(file, "Color8LightBrown = %d\n",  ColorTbl[8]);
		fprintf(file, "Color9DarkBrown = %d\n",   ColorTbl[9]);
		fprintf(file, "Color10LightRed = %d\n",   ColorTbl[10]);
		fprintf(file, "Color11DarkGrey = %d\n",   ColorTbl[11]);
		fprintf(file, "Color12MediumGrey = %d\n", ColorTbl[12]);
		fprintf(file, "Color13LightGreen = %d\n", ColorTbl[13]);
		fprintf(file, "Color14LightBlue = %d\n",  ColorTbl[14]);
		fprintf(file, "Color15LightGrey = %d\n",  ColorTbl[15]);

#ifdef __SYMBIAN32__

		// XY offset
		fprintf(file, "// XY Offset and volume:\n" );
		fprintf(file, "Offset_X = %d\n",          iOffset.iX);
		fprintf(file, "Offset_Y = %d\n",          iOffset.iY);
		fprintf(file, "Volume = %d\n",            iVolume);

#else // __SYMBIAN32__

		fprintf(file, "CIAIRQHack = %s\n", CIAIRQHack ? "TRUE" : "FALSE");
		fprintf(file, "MapSlash = %s\n", MapSlash ? "TRUE" : "FALSE");
		fprintf(file, "ScalingNumerator = %d\n", ScalingNumerator);
		fprintf(file, "ScalingDenominator = %d\n", ScalingDenominator);
		fprintf(file, "ViewPort = %s\n", ViewPort);
		fprintf(file, "DisplayMode = %s\n", DisplayMode);
		fprintf(file, "DisplayType = %s\n", DisplayType == DISPTYPE_WINDOW ? "WINDOW" : "SCREEN");
		fprintf(file, "JoystickSwap = %s\n", JoystickSwap ? "TRUE" : "FALSE");
		fprintf(file, "DoubleScan = %s\n", DoubleScan ? "TRUE" : "FALSE");
		fprintf(file, "HideCursor = %s\n", HideCursor ? "TRUE" : "FALSE");
		fprintf(file, "DirectSound = %s\n", DirectSound ? "TRUE" : "FALSE");
		fprintf(file, "ExclusiveSound = %s\n", ExclusiveSound ? "TRUE" : "FALSE");
		fprintf(file, "AutoPause = %s\n", AutoPause ? "TRUE" : "FALSE");
		fprintf(file, "PrefsAtStartup = %s\n", PrefsAtStartup ? "TRUE" : "FALSE");
		fprintf(file, "SystemMemory = %s\n", SystemMemory ? "TRUE" : "FALSE");
		fprintf(file, "AlwaysCopy = %s\n", AlwaysCopy ? "TRUE" : "FALSE");
		fprintf(file, "SystemKeys = %s\n", SystemKeys ? "TRUE" : "FALSE");
		fprintf(file, "ShowLEDs = %s\n", ShowLEDs ? "TRUE" : "FALSE");

#endif // !__SYMBIAN32__

		fclose(file);

#ifndef __SYMBIAN32__
		ThePrefsOnDisk = *this;	//TODO: use it
#endif

		return true;
	}
	return false;
}


#ifdef __BEOS__
#include "Prefs_Be.i"
#endif

#ifdef AMIGA
#include "Prefs_Amiga.i"
#endif

#ifdef WIN32
#include "Prefs_WIN32.i"
#endif

