/**
 * @file display_epoc32.i - EPOC32 specific display driver
 *
 * (c) 1999-2002 Alfred E. Heggestad
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License version 2 as
 *    published by the Free Software Foundation.
 *
 */

#define BOOL bool
#include <eikon.rsg>


// 0  black
// 1  dark grey
// 2  dark red
// 3  (green)
// 4  dark yellow/green
// 5  light red
// 6  light yellow
// 7  light green
// 8  pink
// 9  dark blue   !!!!
// a  cyan
// b  purple
// c  darker blue !!!!
// d  green/blur ?
// e  light grey
// f  white!


void C64Display::NewPrefs(Prefs *prefs)
	{
	colors_changed = TRUE;
	}


void C64Display::ResetKeyboardState()
	{
	CE32FrodoAppUi* theAppUi = TheC64->iFrodoPtr->iDocPtr->iAppUiPtr;
	memset(theAppUi->keystate, 0, sizeof(theAppUi->keystate));
	memset(theAppUi->key_matrix, 0xff, sizeof(theAppUi->key_matrix));
	memset(theAppUi->rev_matrix, 0xff, sizeof(theAppUi->rev_matrix));
//	joystate = 0xff;
	}


bool C64Display::BuildColorTable()
/**
 * build the color table
 */
	{
	ELOG1(_L8("building color table\n"));

	for (int j = 0; j < 16; j++)
#if COLOR_DEPTH == 4
		{
		uint8 nib_right = TheC64->ThePrefs.ColorTbl[j] & 0x0f;
		colors[j] = nib_right << 4 | nib_right;
		}
#elif COLOR_DEPTH == 8
		colors[j] = TheC64->ThePrefs.ColorTbl[j];
#elif
error
#endif

	// Replicate the physical colors into the rest of the color array.
	for (int k = 16; k < 256; k++)
		colors[k] = colors[k & 0x0f];

	// Tell the VIC all about it;
//	if (!in_constructor)
		TheC64->TheVIC->ReInitColors();

	ELOG1(_L8("Dumping colors: (16 of them)\n"));
	ELOG(for(TInt l = 0; l < 16; l++){ )
		ELOG3(_L8("[%d:%d] "), l, colors[l]);
		ELOG(})
	ELOG1(_L8("\n"));

	return TRUE;
	}


/*

//
 *  Draw LED bar at the bottom of the window
 //

void C64Display::draw_led_bar()
{
}

//
 *  Draw one LED
 //

void C64Display::draw_leds(BOOL force)
{
}

void C64Display::led_rect(int n, RECT &rc, RECT &led)
{
}

*/



C64Display::C64Display(C64 *the_c64) 
/**
 * Display constructor: Create window/screen
 */
	:TheC64(the_c64)
	 ,max_speed(0)
	 ,colors_changed(TRUE)
	 ,colors_depth(4)
	{

	__CHECK_NULL(the_c64);

//	BuildColorTable();
	quit_requested=false;
//	ResetKeyboardState();

	// view Toolbar or not
	TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->ViewToolbar(TheC64->ThePrefs.iShowToolbar);

	CTOR(C64Display);
	}


C64Display::~C64Display()
/**
 * Display destructor
 */
	{
	DTOR(C64Display);
	}


void C64Display::Speedometer(int speed)
/**
 *  Update speedometer
 */
	{
	if ((iSpeedCount++)%50)
		return;
	
	// get peak speed
	if(speed > max_speed)
		max_speed = speed;

	TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->iAppView->DrawSpeed(speed, max_speed);
	}


uint8* C64Display::BitmapBase()
/**
 *  Return pointer to bitmap data
 */
	{
#ifndef __SYMBIAN32__
	static uint8 display[DISPLAY_X * DISPLAY_Y];
#endif

#if defined(__ER6__)
	return (uint8*)TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->iAppView->iMyScreenBuffer;
#else
	return (uint8*)TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->iAppView->iTheBitMap.DataAddress();
#endif
	}


void C64Display::PollKeyboard(uint8 *CIA_key_matrix, uint8 *CIA_rev_matrix, uint8 *joystick)
/**
 *  Freshen keyboard state
 */
	{
	CE32FrodoAppUi* theAppUi = TheC64->iFrodoPtr->iDocPtr->iAppUiPtr;
	memcpy(CIA_key_matrix, theAppUi->key_matrix, sizeof(theAppUi->key_matrix));
	memcpy(CIA_rev_matrix, theAppUi->rev_matrix, sizeof(theAppUi->rev_matrix));

	*joystick = (uint8)theAppUi->iJoyKey;
	}



void C64Display::InitColors(uint8 *array)
/**
 * Allocate C64 colors
 */
	{
	if (colors_depth == 4)
		{
		for (int i = 0; i < 256; i++)
			array[i] = (uint8)colors[i & 0x0f];
		}
	else
		{
		for (int i = 0; i < 256; i++)
			array[i] = TUint8(i & 0x0f);
		}
	}


void C64Display::Update()
	{
	if(colors_changed)
		{
		colors_changed = FALSE;
		BuildColorTable();
		ResetKeyboardState();
		}

#ifdef BACKGND_UPDATE
	TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->iAppView->DrawBuf(TheC64->TheVIC->ec_changed > 0);
#else
	TheC64->iFrodoPtr->iDocPtr->iAppUiPtr->iAppView->DrawBuf(true);
#endif

	}


#define MATRIX(a,b) (((a) << 3) | (b))


int C64Display::VirtKey2C64(const TKeyEvent& aVirtKey, TUint8& aJoyKey)
/**
 * Keyboard handling - see documentation if you dont understand this.

 <pre>

  C64 keyboard matrix:

    Bit 7   6   5   4   3   2   1   0		(b)
  0    CUD  F5  F3  F1  F7 CLR RET DEL
  1    SHL  E   S   Z   4   A   W   3
  2     X   T   F   C   6   D   R   5
  3     V   U   H   B   8   G   Y   7
  4     N   O   K   M   0   J   I   9
  5     ,   @   :   .   -   L   P   +
  6     /   ^   =  SHR HOM  ;   *   
  7    R/S  Q   C= SPC  2  CTL  <-  1

 (a)

 </pre>

 *
 **/
	{
	int result = -1;
	const TUint modifiers = aVirtKey.iModifiers;
	TBool caps = modifiers & EModifierShift;
	const TBool func = modifiers & EModifierFunc;

	// improved keyboard handling

	TChar inChar = aVirtKey.iCode;
	inChar.UpperCase();

	//
	// iCode == 0 means that it's a modifier
	//
	if(aVirtKey.iCode == 0)	// modifier
		{
		if( modifiers & EModifierLeftShift || modifiers & EModifierShift )
		    {
			result = MATRIX(1,7);	// shift left
		    }

		if( modifiers & EModifierLeftShift )
		    {
			result = MATRIX(6,4);	// shift right
		    }

		if( modifiers & EModifierLeftCtrl || modifiers & EModifierRightCtrl || modifiers & EModifierCtrl )
		    {
			result = MATRIX(7,2);	// CTRL
		    }
		}
	else			// normal keys
		switch (inChar)
			{

	//
	// ORing the result with 0x80 means that Shift is also down in C64 world!
	//

		// Function keys:
		//
		// F1 - Fn+A
		// F2 - Fn+S
		// F3 - Fn+D
		// F4 - Fn+F
		// F5 - Fn+G
		// F6 - Fn+H
		// F7 - Fn+J
		// F8 - Fn+K
	
	//
	// Alphabet - if func is true, return F1-F8 for some letters
	//
	case 'A': result = func ? MATRIX(0,4) : MATRIX(1,2); break;
	case 'B': result = MATRIX(3,4); break;
	case 'C': result = MATRIX(2,4); break;
	case 'D': result = func ? MATRIX(0,5) : MATRIX(2,2); break;
	case 'E': result = MATRIX(1,6); break;
	case 'F': result = func ? MATRIX(0,5) | 0x80 : MATRIX(2,5); break;
	case 'G': result = func ? MATRIX(0,6) : MATRIX(3,2); break;
	case 'H': result = func ? MATRIX(0,6) | 0x80 : MATRIX(3,5); break;
	case 'I': result = MATRIX(4,1); break;
	case 'J': result = func ? MATRIX(0,3) : MATRIX(4,2); break;
	case 'K': result = func ? MATRIX(0,3) | 0x80 : MATRIX(4,5); break;
	case 'L': result = MATRIX(5,2); break;
	case 'M': result = MATRIX(4,4); break;
	case 'N': result = MATRIX(4,7); break;
	case 'O': result = MATRIX(4,6); break;
	case 'P': result = MATRIX(5,1); break;
	case 'Q': result = MATRIX(7,6); break;
	case 'R': result = MATRIX(2,1); break;
	case 'S': result = func ? MATRIX(0,4) | 0x80 : MATRIX(1,5); break;
	case 'T': result = MATRIX(2,6); break;
	case 'U': result = MATRIX(3,6); break;
	case 'V': result = MATRIX(3,7); break;
	case 'W': result = MATRIX(1,1); break;
	case 'X': result = MATRIX(2,7); break;
	case 'Y': result = MATRIX(3,1); break;
	case 'Z': result = MATRIX(1,4); break;

	// numeric
	case '0': result = MATRIX(4,3); break;
	case '1': result = MATRIX(7,0); break;
	case '2': result = MATRIX(7,3); break;
	case '3': result = MATRIX(1,0); break;
	case '4': result = MATRIX(1,3); break;
	case '5': result = MATRIX(2,0); break;
	case '6': result = MATRIX(2,3); break;
	case '7': result = MATRIX(3,0); break;
	case '8': result = MATRIX(3,3); break;
	case '9': result = MATRIX(4,0); break;


	////////////// pure keys
	case '+':	result = MATRIX(5,0); caps = EFalse; break;
	case '-':	result = MATRIX(5,3); break;	// ?
	case 0xa3:	result = MATRIX(6,0); caps = EFalse; break;
	case '=':	result = MATRIX(6,5); break;	// ?
	case '*':	result = MATRIX(6,1); caps = EFalse; break;
	case '@':	result = MATRIX(5,6); caps = EFalse; break;
	case '^':	result = MATRIX(6,6); caps = EFalse; break;	// arrow up sign (Fn+6)
	case '_':	result = MATRIX(7,1); break;	// arrow left sign (Fn+1)
	case ':':	result = MATRIX(5,5); caps = EFalse; break;	// colon
	case ';':	result = MATRIX(6,2); break;	// semicolon
	case '.':	result = MATRIX(5,4); break;	// dot
	case ',':	result = MATRIX(5,7); break;	// comma
	case '/':	result = MATRIX(6,7); caps = EFalse; break;	// slash

	case EKeyHome:	result = MATRIX(6,3); break;
	case 0x08:		result = MATRIX(0,0); break;	// DELete key
	case EKeyEscape:result = MATRIX(7,7); break;	// Escape -> RunStop key
	case EKeyEnter:	result = MATRIX(0,1); break;	// Return

	// direction keys
	case EKeyUpArrow:
		result = MATRIX(0,7) | 0x80; 
		break;
	case EKeyDownArrow:		
		result = MATRIX(0,7); 
		break;
	case EKeyLeftArrow:
		result = MATRIX(0,2) | 0x80;
		break;
	case EKeyRightArrow:
		result = MATRIX(0,2);
		break;
	case ' ': // space
		result = MATRIX(7,4); 
		break;

	// control keys
	case EKeyRightShift:	result = MATRIX(6,4); break;	// shift right
	case EKeyLeftShift:		result = MATRIX(1,7); break;	// shift left
	case EKeyLeftCtrl:
	case EKeyRightCtrl:		result = MATRIX(7,2); break;	// CTRL key
	case EKeyTab:			result = MATRIX(7,5); break;	// Commodore key

	// function keys (F1-F8)
	// NB! This is for WINS and future versions only
	case EKeyF1: result = MATRIX(0,4); break;
	case EKeyF2: result = MATRIX(0,4) | 0x80; break; 
	case EKeyF3: result = MATRIX(0,5); break;
	case EKeyF4: result = MATRIX(0,5) | 0x80; break;
	case EKeyF5: result = MATRIX(0,6); break;
	case EKeyF6: result = MATRIX(0,6) | 0x80; break;
	case EKeyF7: result = MATRIX(0,3); break;
	case EKeyF8: result = MATRIX(0,3) | 0x80; break;

	///////////////////  virtual keys:

	case '!':	result = MATRIX(7,0) | 0x80; break;
	case '\"':	result = MATRIX(7,3) | 0x80; break;
	case '#':	result = MATRIX(1,0) | 0x80; break;
	case '$':	result = MATRIX(1,3) | 0x80; break;
	case '%':	result = MATRIX(2,0) | 0x80; break;
	case '&':	result = MATRIX(2,3) | 0x80; break;
	case '\'':	result = MATRIX(3,0) | 0x80; break;
	case '(':	result = MATRIX(3,3) | 0x80; break;
	case ')':	result = MATRIX(4,0) | 0x80; break;
//	case '0':	result = MATRIX(4,3) | 0x80; break;	//Shift+Zero is nothing...

	case '[':	result = MATRIX(5,5) | 0x80; break;	// Shift+colon
	case ']':	result = MATRIX(6,2) | 0x80; break;	// Shift+semicolon
	case '<':	result = MATRIX(5,7) | 0x80; break;	// Shift+comma
	case '>':	result = MATRIX(5,4) | 0x80; break;	// Shift+dot

	case '?':	result = MATRIX(6,7) | 0x80; break;	// Shift+slash


	/// --- Unused keys: --- ///

	case '{':	
	case '}':	
	case '~':	
	case '\\':	
		return -1;

	default:
		ELOG3(_L8("Unknown Key [code=0x%08x, scan=0x%08x]"), aVirtKey.iCode, aVirtKey.iScanCode);
		break;
	}

	// Shift modifier
	if (result != -1 && caps)
		result |= 0x80;

	return result;

	} // end, C64Display::VirtKey2C64


long int ShowRequester(char *a,char *b,char *)
/**
 *  Show a requester (error message)
 */
	{
    TBuf8<32> a8;
    TBuf8<32> b8;
	ELOG3(_L8("ShowRequester: %S %S"), &a8, &b8);
	TBuf<32> _a;
	_a.Copy(a8);
	TBuf<32> _b;
	_b.Copy(b8);

	CEikInfoDialog* info = new CEikInfoDialog(_a, _b);
	if(info)
		{
		TRAPD(ret, info->ExecuteLD(R_EIK_DIALOG_INFO));
		if(ret != KErrNone)
			{
			ELOG2(_L8("hmm. ExecuteLD returned %d"), ret);
			}
		}
	else
		{
		ELOG1(_L8("hmm. could not create CEikInfoDialog"));
		}

	return 1;
	}


int C64Display::BitmapXMod() const
/**
 * Return number of bytes per row
 */
	{
	return DISPLAY_X;
	}


// EOF - display_epoc32.i
