/*
 *  main.cpp - Main program
 *
 *  Frodo (C) 1994-1997 Christian Bauer
 */

#include "sysdeps.h"

#include "main.h"
#include "C64.h"
#include "Display.h"
#include "Prefs.h"
#include "SAM.h"

#ifdef __SYMBIAN32__
#include <e32base.h>
#include "e32frodo.h"
#endif


/**
 * .rom file names
 */
#if defined(__riscos__)
#define BASIC_ROM_FILE	"FrodoRsrc:Basic_ROM"
#define KERNAL_ROM_FILE	"FrodoRsrc:Kernal_ROM"
#define CHAR_ROM_FILE	"FrodoRsrc:Char_ROM"
#define FLOPPY_ROM_FILE	"FrodoRsrc:1541_ROM"
#elif defined(__SYMBIAN32__)
#define BASIC_ROM_FILE	"rom\\basic.rom"
#define KERNAL_ROM_FILE	"rom\\kernal.rom"
#define CHAR_ROM_FILE	"rom\\char.rom"
#define FLOPPY_ROM_FILE	"rom\\1541.rom"
#else
#define BASIC_ROM_FILE	"basic.rom"
#define KERNAL_ROM_FILE	"kernal.rom"
#define CHAR_ROM_FILE	"char.rom"
#define FLOPPY_ROM_FILE	"1541.rom"
#endif


static void MakePath(char* aPathBuf, const char* aFileName)
/**
 * Append aFileName onto the application drive and path.
 */
	{
#ifdef __SYMBIAN32__
	TBuf8<KMaxFileName> driveAndPath;
	driveAndPath.Copy(((CE32FrodoApplication*)CEikonEnv::Static()->EikAppUi()->Application())->GetDriveAndPath());
	TPtrC8 fileName((TText8*)aFileName);
	driveAndPath.Append(fileName);
	driveAndPath.ZeroTerminate();
	Mem::Copy(aPathBuf,driveAndPath.Ptr(),driveAndPath.Length()+1);
#else
	strcpy(aPathBuf, aFileName);
	/// @todo something here
#endif
	}


#ifndef __PSXOS__
bool Frodo::load_rom_files(void)
/**
 *  Load C64.rom files
 */
{
	FILE *file;

	ELOG1(_L8("loading ROMs..."));

	// Load Basic.rom

	char fileName[128];
	MakePath(fileName,BASIC_ROM_FILE);
	if ((file = fopen(fileName, "rb")) != NULL) {
		if (fread(TheC64->Basic, 1, 0x2000, file) != 0x2000) {
			ShowRequester("Can't read 'basic.rom'.", "Quit");
			return false;
		}
		fclose(file);
	} else {
		ShowRequester("Can't find 'basic.rom'.", "Quit");
		return false;
	}

	// Load Kernal ROM
	MakePath(fileName,KERNAL_ROM_FILE);
	if ((file = fopen(fileName, "rb")) != NULL) {
		if (fread(TheC64->Kernal, 1, 0x2000, file) != 0x2000) {
			ShowRequester("Can't read 'Kernal ROM'.", "Quit");
			return false;
		}
		fclose(file);
	} else {
		ShowRequester("Can't find 'Kernal ROM'.", "Quit");
		return false;
	}

	// Load Char.rom
	MakePath(fileName,CHAR_ROM_FILE);
	if ((file = fopen(fileName, "rb")) != NULL) {
		if (fread(TheC64->Char, 1, 0x1000, file) != 0x1000) {
			ShowRequester("Can't read 'Char ROM'.", "Quit");
			return false;
		}
		fclose(file);
	} else {
		ShowRequester("Can't find 'Char ROM'.", "Quit");
		return false;
	}

	// Load 1541.rom
	MakePath(fileName,FLOPPY_ROM_FILE);
	if ((file = fopen(fileName, "rb")) != NULL) {
		if (fread(TheC64->ROM1541, 1, 0x4000, file) != 0x4000) {
			ShowRequester("Can't read '1541 ROM'.", "Quit");
			return false;
		}
		fclose(file);
	} else {
		ShowRequester("Can't find '1541 ROM'.", "Quit");
		return false;
	}

	ELOG1(_L8("[  OK  ]\n"));

	return true;
}
#endif


#ifdef __BEOS__
#include "main_Be.i"
#endif

#ifdef AMIGA
#include "main_Amiga.i"
#endif

#ifdef __unix
#include "main_x.i"
#endif

#ifdef __mac__
#include "main_mac.i"
#endif

#ifdef WIN32
#include "main_WIN32.i"
#endif

#ifdef __riscos__
#include "main_Acorn.i"
#endif

#ifdef __PSXOS__
#include "main_PSX.i"
#endif

#ifdef __SYMBIAN32__
#include "main_epoc32.i"
#endif
